package edu.ntnu.cikmr;

import java.util.Timer;
import java.util.TimerTask;

import edu.ntnu.cikmr.arbitration.Arbitration;
import edu.ntnu.cikmr.gui.LogComponentLog;
import edu.ntnu.cikmr.gui.Main;
import edu.ntnu.cikmr.log.Log;
import edu.ntnu.cikmr.network.Network;
import edu.ntnu.cikmr.scheduling.Scheduler;
import edu.ntnu.cikmr.services.resources.ResourceManagerService;
import jcolibri.extensions.DL.util.OntoBridge;
import javax.swing.event.EventListenerList;
import edu.ntnu.cikmr.arbitration.ConflictListener;
import edu.ntnu.cikmr.arbitration.ConflictEvent;
import edu.ntnu.cikmr.arbitration.cbr.Conflict;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class Framework{
    public static Log log;
    public static Network network;
    public static Arbitration arbitration;
    public static Scheduler scheduler;
    public static ResourceManagerService resources;
    public static Main gui;
    public static OntoBridge ontologies;
    private static EventListenerList listenerList;
    private static Timer timer;
    private static TimerTask timerTaskNetworkConnect = new TimerTask(){ public void run(){ network.connect(); }};
    private static TimerTask timerTaskGUIHide = new TimerTask(){ public void run(){ guiHide(); }};
    private static Boolean started;

    public static void shutdown(){
        synchronized(started){
            if(!started) return;
            started = false;
        }
        fireShuttingDown();
        network.disconnect();
        timer.schedule(timerTaskGUIHide, 1000);
    }

    private static void guiHide(){
        gui.setVisible(false);
        fireShutdown();
        System.exit(0);
    }

    private static void startup(){
        synchronized(started){
            if(started) return;
            started = true;
        }
        listenerList = new EventListenerList();

        log = new LogComponentLog();
        network = Network.getInstance();
        network.getServiceManager().add(arbitration = new Arbitration());

        arbitration.addConflictListener(new ConflictListener(){
            public void possibleConflict(ConflictEvent e){
                conflict(e);
            }
        });

        network.getServiceManager().add(scheduler = new Scheduler());
        network.getServiceManager().add(resources = new ResourceManagerService());
        gui = new Main();
        gui.setVisible(true);
        timer = new Timer();
        timer.schedule(timerTaskNetworkConnect, 1000);

        fireStarted();
    }

    public static boolean isStarted(){
        return started;
    }

    public static void main(String[] args){
        try{
            javax.swing.UIManager.setLookAndFeel(javax.swing.UIManager.getSystemLookAndFeelClassName());
        }catch(Exception ex){}
        started = false;
        startup();
    }

    public static void addFrameworkListener(FrameworkListener l){
        listenerList.add(FrameworkListener.class, l);
    }

    public static void removeFrameworkListener(FrameworkListener l){
        listenerList.remove(FrameworkListener.class, l);
    }

    private static void fireShuttingDown(){
        FrameworkListener[] listeners = listenerList.getListeners(FrameworkListener.class);
        for(int i = listeners.length - 1; i >= 0; i--) listeners[i].frameworkShuttingDown();
    }

    private static void fireShutdown(){
        FrameworkListener[] listeners = listenerList.getListeners(FrameworkListener.class);
        for(int i = listeners.length - 1; i >= 0; i--) listeners[i].frameworkShutdown();
    }

    private static void fireStarted(){
        FrameworkListener[] listeners = listenerList.getListeners(FrameworkListener.class);
        for(int i = listeners.length - 1; i >= 0; i--) listeners[i].frameworkStarted();
    }

    private static void conflict(ConflictEvent e){
        System.out.println("--- Possible conflicts ---");
        System.out.println(e.getPeerName() + " (" + e.getPeerID() + ")");
        System.out.println(e.getContainer().getLocal().getType() + " : " + e.getContainer().getRemote().getType());
        for(int i = 0; i < e.getConflicts().length; i++){
            Conflict c = e.getConflicts()[i];
            System.out.println(c.getCase1() + " : " + c.getCase2() + " (" + c.getSimilarity() + ")");
        }
    }

}
