package edu.ntnu.cikmr.arbitration;

import java.io.IOException;
import java.net.SocketTimeoutException;
import java.util.Enumeration;

import edu.ntnu.cikmr.arbitration.cbr.ArbitrationCBR;
import edu.ntnu.cikmr.arbitration.cbr.Conflict;
import edu.ntnu.cikmr.services.AbstractService;
import edu.ntnu.cikmr.services.ServiceManager;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.peergroup.PeerGroup;
import net.jxta.pipe.PipeService;
import net.jxta.protocol.PipeAdvertisement;
import net.jxta.util.*;
import net.jxta.discovery.DiscoveryListener;
import net.jxta.discovery.DiscoveryEvent;
import edu.ntnu.cikmr.services.peers.Peer;
import edu.ntnu.cikmr.arbitration.workflow.PeerWorkflowContainer;
import edu.ntnu.cikmr.Framework;
import net.jxta.endpoint.Message;
import net.jxta.endpoint.StringMessageElement;
import edu.ntnu.cikmr.arbitration.messages.ArbitrationMessage;
import edu.ntnu.cikmr.arbitration.cbr.Case;
import edu.ntnu.cikmr.arbitration.messages.ConflictMessage;
import edu.ntnu.cikmr.arbitration.messages.SolutionMessage;
import edu.ntnu.cikmr.workflowenactment.Workflow;
import net.jxta.endpoint.ByteArrayMessageElement;
import java.util.Map.Entry;
import java.util.Iterator;


public class Arbitration extends AbstractService{
    private ArbitrationCBR arbitrationCBR;
    private ConflictDiscoverer discoverer;
    private boolean started;
    private JxtaServerPipe serverPipe;
    private ServerPipeRunner serverPipeRunner;
    private PeerGroup peerGroup;

    public Arbitration() {
        arbitrationCBR = new ArbitrationCBR();
        discoverer = new ConflictDiscoverer();
        discoverer.addConfictListener(new ConflictListener(){
            public void possibleConflict(ConflictEvent e){
                Arbitration.this.possibleConflict(e);
            }
        });
    }

    public ArbitrationCBR getCBR(){
        return arbitrationCBR;
    }

    public void init(ServiceManager manager) {
    }

    public void start(PeerGroup peerGroup) throws IOException {
        if(serverPipeRunner != null && serverPipeRunner.run) return;
        started = false;
        this.peerGroup = peerGroup;
        try{
            Enumeration advs = peerGroup.getDiscoveryService().getLocalAdvertisements(DiscoveryService.ADV, "Name", "Arbitration:*");
            while(advs.hasMoreElements()){
                Advertisement adv = (Advertisement)advs.nextElement();
                peerGroup.getDiscoveryService().flushAdvertisement(adv);
            }
        } catch(Exception ex){
            ex.printStackTrace();
        }

        try{
            PipeAdvertisement pipeAdvertisement = PipeUtilities.createNewPipeAdvertisement(peerGroup, PipeService.UnicastType);
            pipeAdvertisement.setName("Arbitration:" + peerGroup.getPeerID().getUniqueValue().toString());

            peerGroup.getDiscoveryService().publish(pipeAdvertisement);
            serverPipe = new JxtaServerPipe(peerGroup, pipeAdvertisement, 20, 2000);

            serverPipeRunner = new ServerPipeRunner();
        }catch(Exception ex){
            ex.printStackTrace();
            throw new IOException("Unable to start server pipe.");
        }
        started = true;
    }

    public void stop() throws IOException {
        if(serverPipeRunner != null && serverPipeRunner.run){
            started = false;
            serverPipeRunner.run = false;

            PipeAdvertisement adv = serverPipe.getPipeAdv();
            try{
                serverPipeRunner.interrupt();
                serverPipeRunner.join();
            } catch(Exception ex){}
            try{
                serverPipe.close();
            } catch(Exception ex){
                ex.printStackTrace();
            }
            serverPipe = null;
            serverPipeRunner = null;
            peerGroup.getDiscoveryService().flushAdvertisement(adv);
        }
    }

    public boolean isStarted() {
        return started;
    }

    private class ServerPipeRunner extends Thread{
        private boolean run;
        private JxtaBiDiPipe pipe;

        public ServerPipeRunner(){
            run = true;
            start();
        }

        public void run(){
            while(run){
                try {
                    pipe = serverPipe.accept();
                } catch (SocketTimeoutException ex) {
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
                if(pipe != null){
                    new ConnectionHandler(pipe);
                    pipe = null;
                }
            }
        }

    }

    private class ConnectionHandler extends Thread{
        private JxtaBiDiPipe pipe;

        private ConnectionHandler(JxtaBiDiPipe pipe){
            this.pipe = pipe;
            start();
        }

        public void run(){
            try {
                Message inbound = pipe.getMessage(10000);
                Message outbound = handleMessage(inbound);
                if(outbound != null) pipe.sendMessage(outbound);
                try {
                    pipe.close();
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }

    }

    private Message handleMessage(Message message){
        ByteArrayMessageElement byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("peerName");
        String peerName = new String(byteArrayElement.getBytes());
        byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("peerID");
        String peerID = new String(byteArrayElement.getBytes());
        byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("command");
        String command = new String(byteArrayElement.getBytes());
        try{
            if(command.startsWith("Arbitration:")){
                byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("xml");
                String xml = new String(byteArrayElement.getBytes());
                ArbitrationMessage arbitrationMessage = ArbitrationMessage.fromXML(xml);
                return handleMessage(arbitrationMessage);
            }
            return null;
        } catch(Exception ex){
            ex.printStackTrace();
            return null;
        }
    }

    private Message handleMessage(ArbitrationMessage message){
        if(message.getSolution().getType().equalsIgnoreCase("wait")){
            return handleWaitMessage(message);
        }else if(message.getSolution().getType().equalsIgnoreCase("waitRemote")){
            return handleWaitRemoteMessage(message);
        }else if(message.getSolution().getType().equalsIgnoreCase("halt")){
            return handleHaltMessage(message);
        }else if(message.getSolution().getType().equalsIgnoreCase("haltRemote")){
            return handleHaltRemoteMessage(message);
        }else if(message.getSolution().getType().equalsIgnoreCase("adapt")){
            return handleAdaptMessage(message);
        }else if(message.getSolution().getType().equalsIgnoreCase("adaptRemote")){
            return handleAdaptRemoteMessage(message);
        }
        return null;
    }

    private Message handleAdaptMessage(ArbitrationMessage message){
        String workflowID = message.getSolution().getParameter("workflowID");
        String pipeID = message.getSolution().getParameter("pipeID");

        Workflow workflow = Framework.scheduler.getWorkflow(workflowID);

        if(workflow == null){
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Failed");
        }else{
            WaitWorkflow.waitOnRemoteWorkflow(workflow, pipeID);
            return createAdaptResponseMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), workflow);
        }
    }

    private Message createAdaptResponseMessage(String peerName, String peerID, Workflow workflow){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:Ok");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("adapt");

        Iterator<Entry<String,String>> iterator = workflow.getGoal().getDefinitionIterator();
        while(iterator.hasNext()){
            Entry<String,String> entry = iterator.next();
            try{
                Double.parseDouble(entry.getValue());
                solution.putParameter(entry.getKey(), entry.getValue());
            } catch(Exception ex){}
        }

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message handleAdaptRemoteMessage(ArbitrationMessage message){
        String workflowID = message.getSolution().getParameter("workflowID");
        String peerID = message.getSolution().getParameter("peerID");
        String peerName = message.getSolution().getParameter("peerName");

        Workflow workflow = Framework.scheduler.getWorkflow(workflowID);

        if(workflow == null){
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Failed");
        }else{
            workflow.addParticipant(peerID, peerName);
            adaptWorkflow(workflow, message);
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Ok");
        }
    }

    private Message handleWaitMessage(ArbitrationMessage message){
        String workflowID = message.getSolution().getParameter("workflowID");
        String peerID = message.getSolution().getParameter("peerID");
        String peerName = message.getSolution().getParameter("peerName");

        Workflow workflow = Framework.scheduler.getWorkflow(workflowID);

        if(workflow == null){
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Failed");
        }else{
            workflow.addParticipant(peerID, peerName);
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Ok");
        }
    }

    private Message handleWaitRemoteMessage(ArbitrationMessage message){
        String workflowID = message.getSolution().getParameter("workflowID");
        String pipeID = message.getSolution().getParameter("pipeID");

        Workflow workflow = Framework.scheduler.getWorkflow(workflowID);

        if(workflow == null){
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Failed");
        }else{
            WaitWorkflow.waitOnRemoteWorkflow(workflow, pipeID);
            return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Ok");
        }
    }

    private Message handleHaltMessage(ArbitrationMessage message){
        return createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Ok");
    }

    private Message handleHaltRemoteMessage(ArbitrationMessage message){
        String workflowID = message.getSolution().getParameter("workflowID");
        Workflow workflow = Framework.scheduler.getWorkflow(workflowID);

        Message response;
        if(workflow == null){
            response = createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Failed");
        }else{
            workflow.halt();
            response = createBasicMessage(peerGroup.getPeerName(), peerGroup.getPeerID().getUniqueValue().toString(), "Arbitration:Ok");
        }
        return response;
    }

    public void addConflictListener(ConflictListener l){
        discoverer.addConfictListener(l);
    }

    public void removeConflictListener(ConflictListener l){
        discoverer.removeConfictListener(l);
    }

    private void possibleConflict(ConflictEvent e){
        Conflict[] conflicts = e.getConflicts();
        int count = 0;
        Conflict c = null;
        Conflict use = null;
        for(int i = 0; i < conflicts.length; i++){
            c = conflicts[i];
            if(c.getSimilarity() == 1){
                count++;
                if(use == null) use = c;
            }
        }

        if(count != 1) use = edu.ntnu.cikmr.gui.SelectSolution.selectSolution(null, e);

        if(use != null){

            if(use.getSimilarity() < 1){
                use.setCase1(e.getContainer().getCase1());
                use.setCase2(e.getContainer().getCase2());
                use.setLocation(e.getContainer().getLocation());
                use.setParticipants(e.getContainer().getParticipants());

                arbitrationCBR.learnCase(use);

            }else{

            }
            // Adapt the selected conflict to match the case


            String solution = use.getSolution() == null ? "" : use.getSolution();
            if(solution.equalsIgnoreCase("Halt_case_1")){
                haltWorkflow(e, use);
            }else if(solution.equalsIgnoreCase("Halt_case_2")){
                haltRemoteWorkflow(e, use);
            }else if(solution.equalsIgnoreCase("Prioritize_case_1")){
                waitRemoteWorkflow(e, use);
            }else if(solution.equalsIgnoreCase("Prioritize_case_2")){
                waitWorkflow(e, use);
            }else if(solution.equalsIgnoreCase("Use_case_1")){
                adaptWorkflow(e, use);
            }else if(solution.equalsIgnoreCase("Use_case_2")){
                adaptRemoteWorkflow(e, use);
            }

        }


    }

    private Message communicate(String peerID, Message message, boolean waitForResponse){
        Message responseMessage = null;
        JxtaBiDiPipe pipe = null;
        try {
            PeerGroup peerGroup = Framework.network.getPeerGroup();

            PipeAdvertisement pipeAdvertisement = getArbitrationPipeAdvertisement(peerGroup, peerID);

            pipe = new JxtaBiDiPipe();
            pipe.setReliable(true);
            pipe.connect(Framework.network.getPeerGroup(), pipeAdvertisement, 5000);
            pipe.sendMessage(message);
            if(waitForResponse) responseMessage = pipe.getMessage(10000);
            // Close pipe
            pipe.close();
            // Extract response from message
            return responseMessage;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        try{
            pipe.close();
        }catch(Exception ex){}
        return responseMessage;
    }

    private PipeAdvertisement getArbitrationPipeAdvertisement(PeerGroup peerGroup, String peerID) throws Exception{
        PipeAdvertisementFinder finder = new PipeAdvertisementFinder(peerGroup, peerID);
        return finder.get();
    }

    private class PipeAdvertisementFinder extends Thread implements DiscoveryListener{
        PeerGroup peerGroup;
        String peerID;
        PipeAdvertisement pipeAdvertisement;
        int wait = 10000;
        Object lock;

        private PipeAdvertisementFinder(PeerGroup peerGroup, String peerID){
            this.peerGroup = peerGroup;
            this.peerID = peerID;
            lock = new Object();
            pipeAdvertisement = null;
            start();
        }

        public void run(){
            synchronized(lock){
                peerGroup.getDiscoveryService().getRemoteAdvertisements("urn:jxta:" + peerID, DiscoveryService.ADV, "Name", "Arbitration:" + peerID, 10, this);
                try{
                    Thread.sleep(wait);
                } catch(Exception ex){}
                lock.notify();
            }
        }

        public void discoveryEvent(DiscoveryEvent discoveryEvent){
            Enumeration enumeration = discoveryEvent.getSearchResults();
            while(enumeration.hasMoreElements()){
                try{
                    PipeAdvertisement adv = (PipeAdvertisement)enumeration.nextElement();
                    if(adv.getName().equalsIgnoreCase("Arbitration:" + peerID)){
                        pipeAdvertisement = adv;
                        try{
                            this.interrupt();
                        } catch(Exception ex){}
                        return;
                    }
                } catch(Exception ex){}
            }
        }

        public PipeAdvertisement get() throws Exception{
            synchronized(lock){
                try{
                    lock.wait();
                } catch(Exception ex){}
                if(pipeAdvertisement == null) throw new Exception("Unable to find arbitration pipe.");
                return pipeAdvertisement;
            }
        }

    }

    public void haltWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Workflow workflow = Framework.scheduler.getWorkflow(e.getContainer().getLocal().getWorkflowID());
        if(workflow == null) return;

        workflow.halt();

        Message message = createHaltMessage(peerName, peerID, conflict);
        Message response = communicate(e.getPeerID(), message, true);
        // We do not care about the response, so do nothing
    }

    public void haltRemoteWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Message message = createHaltRemoteMessage(peerName, peerID, conflict, e.getContainer().getRemote().getWorkflowID());
        Message response = communicate(e.getPeerID(), message, true);
        // We do not care about the response, so do nothing
    }

    private void waitWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Workflow workflow = Framework.scheduler.getWorkflow(e.getContainer().getLocal().getWorkflowID());
        if(workflow == null) return;

        Message message = createWaitMessage(peerName, peerID, conflict, e.getContainer().getRemote().getWorkflowID());
        Message response = communicate(e.getPeerID(), message, true);

        WaitWorkflow.waitOnRemoteWorkflow(workflow, e.getContainer().getRemote().getPipeID());

        // We do not care about the response, so do nothing
    }

    private void waitRemoteWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Workflow workflow = Framework.scheduler.getWorkflow(e.getContainer().getLocal().getWorkflowID());
        if(workflow == null) return;
        workflow.addParticipant(e.getPeerID(), e.getPeerName());

        Message message = createWaitRemoteMessage(peerName, peerID, conflict, e.getContainer().getRemote().getWorkflowID(), e.getContainer().getLocal().getPipeID());
        Message response = communicate(e.getPeerID(), message, true);

        // We do not care about the response, so do nothing
    }

    private void adaptRemoteWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Workflow workflow = Framework.scheduler.getWorkflow(e.getContainer().getLocal().getWorkflowID());
        if(workflow == null) return;

        Message message = createAdaptRemoteMessage(peerName, peerID, conflict, workflow, e.getContainer().getRemote().getWorkflowID());
        Message response = communicate(e.getPeerID(), message, true);

        WaitWorkflow.waitOnRemoteWorkflow(workflow, e.getContainer().getRemote().getPipeID());
    }

    private void adaptWorkflow(ConflictEvent e, Conflict conflict){
        String peerName = peerGroup.getPeerName();
        String peerID = peerGroup.getPeerID().getUniqueValue().toString();

        Workflow workflow = Framework.scheduler.getWorkflow(e.getContainer().getLocal().getWorkflowID());
        if(workflow == null) return;
        workflow.addParticipant(e.getPeerID(), e.getPeerName());

        Message message = createAdaptMessage(peerName, peerID, conflict, workflow.getServerPipeID(), e.getContainer().getRemote().getWorkflowID());
        Message response = communicate(e.getPeerID(), message, true);
        adaptWorkflow(workflow, response);
    }

    private void adaptWorkflow(Workflow workflow, Message message){
        ByteArrayMessageElement byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("peerName");
        String peerName = new String(byteArrayElement.getBytes());
        byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("peerID");
        String peerID = new String(byteArrayElement.getBytes());
        byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("command");
        String command = new String(byteArrayElement.getBytes());
        try{
            if(command.equalsIgnoreCase("Arbitration:Ok")){
                byteArrayElement = (ByteArrayMessageElement)message.getMessageElement("xml");
                String xml = new String(byteArrayElement.getBytes());
                ArbitrationMessage arbitrationMessage = ArbitrationMessage.fromXML(xml);
                adaptWorkflow(workflow, arbitrationMessage);
            }
        } catch(Exception ex){
            ex.printStackTrace();
        }
    }

    private void adaptWorkflow(Workflow workflow, ArbitrationMessage message){
        Iterator<Entry<String,String>> params = message.getSolution().getParameterIterator();
        while(params.hasNext()){
            Entry<String,String> entry = params.next();
            try{
                if(workflow.getGoal().hasDefinition(entry.getKey())){
                    double remoteValue = Double.parseDouble(entry.getValue());
                    double localValue = Double.parseDouble(workflow.getGoal().getDefinition(entry.getKey()));
                    double newValue = (remoteValue + localValue) / 2;
                    workflow.getGoal().setDefinition(entry.getKey(), newValue + "");
                }else{

                }
            } catch(Exception ex){}
        }
    }

    private Message createBasicMessage(String peerName, String peerID, String command){
        Message message = new Message();

        StringMessageElement nameMessageElement = new StringMessageElement("peerName", peerName, null);
        message.addMessageElement(nameMessageElement);

        StringMessageElement idMessageElement = new StringMessageElement("peerID", peerID, null);
        message.addMessageElement(idMessageElement);

        if(command != null){
            StringMessageElement commandMessageElement = new StringMessageElement("command", command, null);
            message.addMessageElement(commandMessageElement);
        }

        return message;
    }

    private Message createAdaptMessage(String peerName, String peerID, Case conflict, String pipeID, String remoteWorkflowID){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:Adapt");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("adapt");
        solution.putParameter("pipeID", pipeID);
        solution.putParameter("workflowID", remoteWorkflowID);

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message createAdaptRemoteMessage(String peerName, String peerID, Case conflict, Workflow workflow, String remoteWorkflowID){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:AdaptRemote");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("adaptRemote");
        solution.putParameter("peerName", peerName);
        solution.putParameter("peerID", peerID);
        solution.putParameter("workflowID", remoteWorkflowID);

        Iterator<Entry<String,String>> iterator = workflow.getGoal().getDefinitionIterator();
        while(iterator.hasNext()){
            Entry<String,String> entry = iterator.next();
            try{
                Double.parseDouble(entry.getValue());
                solution.putParameter(entry.getKey(), entry.getValue());
            } catch(Exception ex){}
        }

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message createWaitMessage(String peerName, String peerID, Case conflict, String workflowID){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:Wait");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("wait");
        solution.putParameter("peerID", peerID);
        solution.putParameter("peerName", peerName);
        solution.putParameter("workflowID", workflowID);

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message createWaitRemoteMessage(String peerName, String peerID, Case conflict, String workflowID, String pipeID){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:WaitRemote");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("waitRemote");
        solution.putParameter("pipeID", pipeID);
        solution.putParameter("workflowID", workflowID);

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message createHaltMessage(String peerName, String peerID, Case conflict){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:Halt");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("halt");

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

    private Message createHaltRemoteMessage(String peerName, String peerID, Case conflict, String workflowID){
        Message message = createBasicMessage(peerName, peerID, "Arbitration:HaltRemote");

        SolutionMessage solution = new SolutionMessage();
        solution.setType("haltRemote");
        solution.putParameter("workflowID", workflowID);

        ArbitrationMessage arbitrationMessage = new ArbitrationMessage();
        arbitrationMessage.setPeerName(peerName);
        arbitrationMessage.setPeerID(peerID);
        arbitrationMessage.setConflict(ConflictMessage.fromCase(conflict));
        arbitrationMessage.setSolution(solution);

        StringMessageElement xmlMessageElement = new StringMessageElement("xml", arbitrationMessage.toXML(), null);
        message.addMessageElement(xmlMessageElement);

        return message;
    }

}
