package edu.ntnu.cikmr.arbitration;

import edu.ntnu.cikmr.workflowenactment.Workflow;
import edu.ntnu.cikmr.workflowenactment.WorkflowLifecycleListener;
import edu.ntnu.cikmr.workflowenactment.WorkflowEvent;
import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.scheduling.SchedulerListener;

public class WaitWorkflow implements WorkflowLifecycleListener, SchedulerListener{
    private static final String WAIT_WORKFLOW_XML = "<goal/>";
    private Workflow localWorkflow;
    private Workflow waitWorkflow;
    private boolean running;

    private void start(Workflow waitingWorkflow, String pipeID){
        localWorkflow = waitingWorkflow;
        if(localWorkflow.isExecuting()) localWorkflow.halt();
        localWorkflow.addWorkflowLifecycleListener(this);

        waitWorkflow = Framework.scheduler.addWorkflow(WAIT_WORKFLOW_XML);
        waitWorkflow.addWorkflowLifecycleListener(this);
        waitWorkflow.remoteConnect(pipeID);
        running = true;
    }

    private void stop(){
        running = false;
        waitWorkflow.removeWorkflowLifecycleListener(this);
        Framework.scheduler.removeWorkflow(waitWorkflow);

        localWorkflow.removeWorkflowLifecycleListener(this);
        if(!localWorkflow.isExecuting()) localWorkflow.execute();
    }

    public Workflow getLocalWorkflow(){
        return localWorkflow;
    }

    public Workflow getWaitWorkflow(){
        return waitWorkflow;
    }

    public void workflowStarted(WorkflowEvent e){
        if(!running) return;
        if(e.getWorkflow() == waitWorkflow){

        }else{
            stop();
        }
    }

    public void workflowStopped(WorkflowEvent e){
        if(!running) return;
        if(e.getWorkflow() == waitWorkflow){
            stop();
        }else{

        }
    }

    public void workflowFinished(WorkflowEvent e){
        if(!running) return;
        if(e.getWorkflow() == waitWorkflow){
            stop();
        }else{

        }
    }

    public void workflowExecutionMoved(WorkflowEvent e){
        if(!running) return;
        if(e.getWorkflow() == waitWorkflow){
            if(waitWorkflow.isExecutedLocally()) stop();
        }else{
            stop();
        }
    }

    public void workflowParticipantsChanged(WorkflowEvent e){

    }

    public void workflowAdded(WorkflowEvent e){
    }

    public void workflowRemoved(WorkflowEvent e){
        if(!running) return;
        if(e.getWorkflow() == localWorkflow || e.getWorkflow() == waitWorkflow) stop();
    }

    public static void waitOnRemoteWorkflow(Workflow localWorkflow, String remoteWorkflowPipeID){
        WaitWorkflow w = new WaitWorkflow();
        w.start(localWorkflow, remoteWorkflowPipeID);
    }

}
