package edu.ntnu.cikmr.arbitration.cbr;

import java.io.File;
import java.util.*;

import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.FrameworkListener;
import jcolibri.cbrcase.CBRQuery;
import jcolibri.cbrcase.CaseEvaluation;
import jcolibri.cbrcore.*;
import jcolibri.cbrcore.exception.ExecutionException;
import jcolibri.cbrcore.packagemanager.PackageManager;
import jcolibri.extensions.DL.util.OntoBridge;
import jcolibri.extensions.DL.util.OntoBridgeSingleton;
import jcolibri.cbrcase.CBRCase;
import jcolibri.cbrcase.IndividualRelation;
import jcolibri.cbrcase.Individual;

public class ArbitrationCBR implements FrameworkListener{
    private boolean isInitialized;
    private CBRCore core;

    public CBRTask obtainCasesTask = null;
    public CBRTask obtainQueryTask = null;
    public CBRTask selectWorkingCasesTask = null;
    public CBRTask computeSimilarityTask = null;
    public CBRTask selectBestCasesTask = null;
    public CBRTask prepareCasesForAdaptionTask = null;
    public CBRTask atomicReuseTask = null;
    public CBRTask storeCasesTask = null;
    public CBRTask closeConnectorTask = null;

    public ArrayList<CBRTask> preCycleTaskList = new ArrayList<CBRTask>();
    public ArrayList<CBRTask> cycleTaskList = new ArrayList<CBRTask>();
    public ArrayList<CBRTask> postCycleTaskList = new ArrayList<CBRTask>();
    private static final double threshold = 0.80;


    public ArbitrationCBR(){
        isInitialized = false;
        try {

            core = new CBRCore("Arbitration");
            PackageManager.getInstance().getPackageByName("Core").setActive(true);
            PackageManager.getInstance().getPackageByName("Description Logic Extension").setActive(true);
            PackageManager.getInstance().getPackageByName("User Components").setActive(true);
            core.init();
            setCBRApplicationConfiguration();
            isInitialized = true;

            Framework.addFrameworkListener(this);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Returns wheter or not this arbitration cbr context was able to perform its
     * precycle and therefor was successfully initialized.
     *
     * @return true if successfull, false if not
     */
    public boolean isInitialized(){
        return isInitialized;
    }

    /**
     * Init method of the application. Will invoke several methods
     * to leave ready the application.
     */
    public void open(){
        try {
            executeTasks(preCycleTaskList);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void close(){
        try{
            executeTasks(postCycleTaskList);
        } catch(Exception ex){
            ex.printStackTrace();
        }
    }

    /**
     * Resets the CBR application
     */
    public void reset() {
        try {
            core.resetContext();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Describes the context (current state) of the CBR Application
     * @returns String describing current context.
     */
    public CBRContext getContext() {
        return core.getContext();
    }

    /**
     * This method will be invoked to run some tasks of the CBR Application
     * @return CBRContext, result of the CBR application execution
     */
    public CBRContext executeTasks(List<CBRTask> taskList) throws ExecutionException {
        core.executeTasks((CBRTask[]) taskList.toArray(new CBRTask[taskList.size()]));
        return core.getContext();
    }

    public CBRContext executeTask(CBRTask task) throws ExecutionException{
        core.executeTask(task);
        return core.getContext();
    }

    /**
     * This method holds the automatic code generated for your application
     */
    public final void setCBRApplicationConfiguration() {
        CBRTask task;
        CBRMethod method;
        HashMap<String,Object> parametersMap;
        try {

            // Configurate ontologies
            File ontologyFile = new File("arbitration/arbitration.owl");;
            OntoBridgeSingleton.setFile(ontologyFile.toURI().toString());
            OntoBridgeSingleton.setType(OntoBridge.ReasonerType.PELLET);
            // Instanciate ontology bridge
            Framework.ontologies = OntoBridgeSingleton.getOntoBridge();

            /****************************/
            /* PreCycle Tasks & Methods */
            /****************************/

            //-- PreCycle ------------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.PreCycleMethod","preCycleMethod");

            task = TasksInstanceRegistry.getInstance().createInstance("PreCycle","preCycleTask");
            core.resolveTaskWithMethod(task, method);

            preCycleTaskList.add(task);

            //-- Obtain cases --------------------------------------------------
            parametersMap = new HashMap<String, Object>();
            parametersMap.put("Connector", "arbitration/connector.xml");

            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.LoadCaseBaseMethod","loadCaseBaseMethod");
            method.setParameters(parametersMap);

            obtainCasesTask = TasksInstanceRegistry.getInstance().createInstance("Obtain cases task","obtainCasesTask");
            core.resolveTaskWithMethod(obtainCasesTask, method);

            preCycleTaskList.add(obtainCasesTask);
            //------------------------------------------------------------------


            /****************************/
            /* Cycle Tasks & Methods */
            /****************************/

            //-- Cycle ---------------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.CBRMethod","CBRMethod");

            task = TasksInstanceRegistry.getInstance().createInstance("CBR Cycle","CBRCycleTask");
            core.resolveTaskWithMethod(task, method);

            cycleTaskList.add(task);

            //-- Obtain query --------------------------------------------------
            parametersMap = new HashMap<String, Object>();
            parametersMap.put("Case Structure", "arbitration/caseStructure.xml");

            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.ConfigureQueryMethod","configureQueryMethod");
            method.setParameters(parametersMap);

            obtainQueryTask = TasksInstanceRegistry.getInstance().createInstance("Obtain query task","obtainQueryTask");
            core.resolveTaskWithMethod(obtainQueryTask, method);

            cycleTaskList.add(obtainQueryTask);

            //-- Retrieve ------------------------------------------------------

            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.RetrieveComputationalMethod","retrieveComputationalMethod");

            task = TasksInstanceRegistry.getInstance().createInstance("Retrieve Task","retrieveTask");
            core.resolveTaskWithMethod(task, method);

            cycleTaskList.add(task);

            //-- Select working cases ------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("edu.ntnu.cikmr.arbitration.cbr.SelectAllCasesMethod","selectAllCasesMethod");

            selectWorkingCasesTask = TasksInstanceRegistry.getInstance().createInstance("Select working cases task","selectWorkingCasesTask");
            core.resolveTaskWithMethod(selectWorkingCasesTask, method);

            cycleTaskList.add(selectWorkingCasesTask);

            //-- Compute similarity --------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.NumericSimComputationMethod","numericSimComputationMethod");

            computeSimilarityTask = TasksInstanceRegistry.getInstance().createInstance("Compute similarity task","computeSimilarityTask");
            core.resolveTaskWithMethod(computeSimilarityTask, method);

            cycleTaskList.add(computeSimilarityTask);

            //-- Select best ---------------------------------------------------
            //method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.SelectBestCaseMethod","selectBestCaseMethod");
            method = MethodsInstanceRegistry.getInstance().createInstance("edu.ntnu.cikmr.arbitration.cbr.SelectBestCasesMethod","selectBestCaseMethod");

            selectBestCasesTask = TasksInstanceRegistry.getInstance().createInstance("Select best task","selectBestTask");
            core.resolveTaskWithMethod(selectBestCasesTask, method);

            cycleTaskList.add(selectBestCasesTask);

            //-- Reuse ---------------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.ReuseComputationalMethod","reuseComputationalMethod");

            task = TasksInstanceRegistry.getInstance().createInstance("Reuse Task","reuseTask");
            core.resolveTaskWithMethod(task, method);

            cycleTaskList.add(task);

            //-- Prepare cases for adaption ------------------------------------
            parametersMap = new HashMap<String,Object>();
            parametersMap.put("Case Structure", "arbitration/caseStructure.xml");

            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.CopyCasesforAdaptationMethod","copyCasesforAdaptationMethod");
            method.setParameters(parametersMap);

            prepareCasesForAdaptionTask = TasksInstanceRegistry.getInstance().createInstance("Prepare Cases for Adaptation Task","prepareCasesForAdaptationTask");
            core.resolveTaskWithMethod(prepareCasesForAdaptionTask, method);

            cycleTaskList.add(prepareCasesForAdaptionTask);

            //-- Atomic reuse --------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.CombineQueryAndCasesMethod","combineQueryAndCasesMethod");

            atomicReuseTask = TasksInstanceRegistry.getInstance().createInstance("Atomic Reuse Task","atomicReuseTask");
            core.resolveTaskWithMethod(atomicReuseTask, method);

            cycleTaskList.add(atomicReuseTask);

            //-- Retain --------------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.RetainComputationalMethod","retainComputationalMethod");

            task = TasksInstanceRegistry.getInstance().createInstance("Retain Task","retainTask");
            core.resolveTaskWithMethod(task, method);

            cycleTaskList.add(task);

            //-- Store cases ---------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.StoreCasesMethod","storeCasesMethod");

            storeCasesTask = TasksInstanceRegistry.getInstance().createInstance("Store cases task","storeCasesTask");
            core.resolveTaskWithMethod(storeCasesTask, method);

            cycleTaskList.add(storeCasesTask);
            //------------------------------------------------------------------


            /****************************/
            /* PostCycle Tasks & Methods */
            /****************************/

            //-- Post cycle ----------------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.PostCycleMethod","postCycleMethod");
            task = TasksInstanceRegistry.getInstance().createInstance("PostCycle","postCycle");
            core.resolveTaskWithMethod(task, method);

            postCycleTaskList.add(task);

            //-- Close connector -----------------------------------------------
            method = MethodsInstanceRegistry.getInstance().createInstance("jcolibri.method.CloseConnectorMethod","closeConnectorMethod");

            closeConnectorTask = TasksInstanceRegistry.getInstance().createInstance("Close connector task","closeConnectorTask");
            core.resolveTaskWithMethod(closeConnectorTask, method);

            postCycleTaskList.add(closeConnectorTask);
            //------------------------------------------------------------------

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void frameworkStarted(){
        open();
    }

    public void frameworkShuttingDown(){
    }

    public void frameworkShutdown(){
        close();
    }

    public synchronized Conflict[] checkForConflict(Case c){
        CBRQuery query = CaseUtils.createQuery(c);
        core.setQuery(query);
        try{
            executeTask(selectWorkingCasesTask);
            if(core.getContext().getCases() == null || core.getContext().getCases().size() == 0) return null;
            executeTask(computeSimilarityTask);
            Conflict[] conflicts = getConflicts();
            core.getContext().setCases(null);
            return conflicts;
        } catch(Exception ex){
            ex.printStackTrace();
            return null;
        }
    }

    public Conflict[] getConflicts(){
        List<Conflict> cases = new ArrayList<Conflict>();
        CBRContext context = core.getContext();

        if(context.getEvaluatedCases() == null || context.getEvaluatedCases().size() == 0) return null;
        Iterator i = context.getEvaluatedCases().iterator();

        while(i.hasNext()){
            CaseEvaluation eval = (CaseEvaluation)i.next();

            double evalVal = 0;
            if(eval.getEvaluation() instanceof Double) evalVal = ((Double)eval.getEvaluation()).doubleValue();
            if(evalVal >= threshold && isLocationFullfilled(eval)){
                Conflict c = CaseUtils.convertConflict(eval.getCase(), evalVal);
                if(c != null) cases.add(c);
            }
        }
        if(cases.size() == 0){
            return null;
        }else{
            return cases.toArray(new Conflict[cases.size()]);
        }
    }

    private boolean isLocationFullfilled(CaseEvaluation eval){
        CBRQuery q = getContext().getQuery();
        IndividualRelation individualRelation = q.getDescription().getRelation("has-Description.Location");
        Individual individual = individualRelation.getTarget();
        double queryLocation = (Double)individual.getValue();

        Case e = CaseUtils.convert(eval.getCase());

        return queryLocation >= e.getLocation();
    }

    public synchronized void learnCase(Case c){
        List<CBRCase> list = new ArrayList<CBRCase>();
        CBRCase cbrCase = CaseUtils.convert(c);

        list.add(cbrCase);
        getContext().setCases(list);
        try{
            executeTask(storeCasesTask);
        } catch(Exception ex){
            ex.printStackTrace();
        }
    }

}
