package edu.ntnu.cikmr.arbitration.cbr;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.ontologies.OntologyUtils;
import jcolibri.cbrcase.*;
import jcolibri.extensions.DL.datatypes.ConceptType;
import jcolibri.util.CaseCreatorUtils;

public class CaseUtils{
    public static final String CASE_STRUCTURE_FILE = "arbitration/caseStructure.xml";

    public static CBRQuery createQuery(Case c){

        CBRQuery caseQuery = CaseCreatorUtils.createQuery(CASE_STRUCTURE_FILE);

        IndividualRelation individualRelation = null;
        OntClass concept = null;
        Individual individual = null;

        individualRelation = caseQuery.getDescription().getRelation("has-Description.Case_1");
        individualRelation.setWeight(1);
        individual = Framework.ontologies.getInstanceByLocalName(c.getCase1());
        if(individual != null){
            concept = OntologyUtils.findOwner("Case_1", individual.getLocalName());
            if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
        }

        individualRelation = caseQuery.getDescription().getRelation("has-Description.Case_2");
        individualRelation.setWeight(1);
        individual = Framework.ontologies.getInstanceByLocalName(c.getCase2());
        if(individual != null){
            concept = OntologyUtils.findOwner("Case_2", individual.getLocalName());
            if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
        }

        individualRelation = caseQuery.getDescription().getRelation("has-Description.Location");
        individualRelation.getTarget().setValue(c.getLocation());
        individualRelation.setWeight(1);

        individualRelation = caseQuery.getDescription().getRelation("has-Description.Participants");
        individualRelation.getTarget().setValue(c.getParticipants());
        individualRelation.setWeight(1);

        return caseQuery;
    }

    public static CBRCaseRecord convert(Case c){

        CBRCaseRecord caseRecord = CaseCreatorUtils.createCase(CASE_STRUCTURE_FILE);
        CaseCreatorUtils.setCaseName(caseRecord, c.getId());

        IndividualRelation individualRelation = null;
        OntClass concept = null;
        Individual individual = null;

        individualRelation = caseRecord.getDescription().getRelation("has-Description.Case_1");
        individual = Framework.ontologies.getInstanceByLocalName(c.getCase1());
        if(individual != null){
            concept = OntologyUtils.findOwner("Case_1", individual.getLocalName());
            if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
        }

        individualRelation = caseRecord.getDescription().getRelation("has-Description.Case_2");
        individual = Framework.ontologies.getInstanceByLocalName(c.getCase2());
        if(individual != null){
            concept = OntologyUtils.findOwner("Case_2", individual.getLocalName());
            if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
        }

        individualRelation = caseRecord.getDescription().getRelation("has-Description.Location");
        individualRelation.getTarget().setValue(c.getLocation());

        individualRelation = caseRecord.getDescription().getRelation("has-Description.Participants");
        individualRelation.getTarget().setValue(c.getParticipants());

        if(c.getSolution() != null){
            individualRelation = caseRecord.getSolution().getRelation("has-Solution.Solution");
            individual = Framework.ontologies.getInstanceByLocalName(c.getSolution());
            if(individual != null){
                concept = OntologyUtils.findOwner("Solution", individual.getLocalName());
                if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
            }
        }

        if(c.getProperty() != null){
            individualRelation = caseRecord.getSolution().getRelation("has-Solution.Property");
            individualRelation.getTarget().setValue(c.getProperty());
        }

        return caseRecord;
    }

    public static Case convert(CBRCase cbrCase){
        try {
            Case c = new Case();

            IndividualRelation individualRelation = null;
            jcolibri.cbrcase.Individual individual = null;

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_1");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setCase1(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_2");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setCase2(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Location");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setLocation((Double)individual.getValue());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Participants");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setParticipants((Double)individual.getValue());

            individualRelation = cbrCase.getSolution().getRelation("has-Solution.Solution");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setSolution(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getSolution().getRelation("has-Solution.Property");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setProperty(individual.getValue().toString());

            return c;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    public static Conflict convertConflict(CBRCase cbrCase, double similarity){
        try {

            Conflict c = new Conflict();

            IndividualRelation individualRelation = null;
            jcolibri.cbrcase.Individual individual = null;

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_1");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setCase1(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_2");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setCase2(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Location");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setLocation((Double)individual.getValue());

            individualRelation = cbrCase.getDescription().getRelation("has-Description.Participants");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setLocation((Double)individual.getValue());

            individualRelation = cbrCase.getSolution().getRelation("has-Solution.Solution");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setSolution(((ConceptType)individual.getValue()).getOntIndividual().getLocalName());

            individualRelation = cbrCase.getSolution().getRelation("has-Solution.Property");
            individual = individualRelation.getTarget();
            if(individual.getValue() != null) c.setProperty(individual.getValue().toString());

            c.setSimilarity(similarity);
            return c;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

}
