package edu.ntnu.cikmr.arbitration.cbr;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.ontologies.OntologyUtils;
import jcolibri.cbrcase.*;
import jcolibri.cbrcore.exception.InitializingException;
import jcolibri.extensions.DL.datatypes.ConceptType;
import jcolibri.util.CaseCreatorUtils;

public class Connector implements jcolibri.cbrcase.Connector {
    private String databaseFile;
    private String caseStructureFile;
    private int caseNumber;

    public Connector(){
        databaseFile = "arbitration/repository.csv";
        caseStructureFile = "arbitration/caseStructure.xml";
    }

    /**
     * close
     *
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public void close() {
    }

    /**
     * deleteCases
     *
     * @param collection Collection
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public synchronized void deleteCases(Collection collection) {
        Collection cases = retrieveAllCases();

        if(collection == null || collection.size() == 0 || cases == null || cases.size() == 0) return;

        int[] deleteIds = getCaseIds(collection);

        InputStream inputStream = null;
        BufferedWriter bufferedWriter = null;
        BufferedReader bufferedReader = null;
        try {
            File file = new File(databaseFile);
            int fileSize = (int)file.length();
            byte[] buffer = new byte[fileSize];
            inputStream = new FileInputStream(file);
            inputStream.read(buffer);
            inputStream.close();

            bufferedReader = new BufferedReader(new InputStreamReader(new ByteArrayInputStream(buffer)));
            bufferedWriter = new BufferedWriter(new FileWriter(file, false));
            String line = null;
            while((line = bufferedReader.readLine()) != null){
                if(!skipLine(line, deleteIds)){
                    bufferedWriter.write(line);
                    bufferedWriter.newLine();
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }finally{
            try {
                inputStream.close();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            try {
                bufferedReader.close();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            try {
                bufferedWriter.flush();
                bufferedWriter.close();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    private int[] getCaseIds(Collection collection){
        int[] ids = new int[collection.size()];
        int count = 0;
        Iterator iterator = collection.iterator();
        while(iterator.hasNext()){
            CBRCase cbrCase = (CBRCase)iterator.next();
            int id = 0;
            try{
                id = Integer.parseInt((String)cbrCase.getValue(), 10);
            }catch(Exception ex){
                continue;
            }
            ids[count++] = id;
        }
        Arrays.sort(ids);
        return ids;
    }

    private boolean skipLine(String line, int[] skipIds){
        if(line == null) return true;
        line = line.trim();
        if(line.length() == 0 || line.startsWith("#")) return false;

        String[] elements = line.split(";");
        int id = 0;
        try {
            id = Integer.parseInt(elements[0], 10);
        } catch (Exception ex) {
            // Skip lines that we are unable to decode
            return true;
        }
        return Arrays.binarySearch(skipIds, id) >= 0;
    }

    /**
     * init
     *
     * @param properties Properties
     * @throws InitializingException
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public void init(Properties properties) throws InitializingException {
    }

    /**
     * retrieveAllCases
     *
     * @return Collection
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public synchronized Collection retrieveAllCases() {
        List cases = new ArrayList();
        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new FileReader(databaseFile));

            String line = null;
            while((line = reader.readLine()) != null){
                try{
                    line = line.trim();
                    // Skip lines that are empty or that are commented out
                    if(line.length() == 0 || line.startsWith("#")) continue;
                    String[] elements = line.split(";");

                    CBRCaseRecord caseRecord = CaseCreatorUtils.createCase(caseStructureFile);
                    CaseCreatorUtils.setCaseName(caseRecord, elements[0]);

                    try {
                        caseNumber = Integer.parseInt(elements[0], 10);
                    } catch (Exception ex) {}

                    IndividualRelation individualRelation = null;
                    OntClass concept = null;
                    Individual individual = null;

                    individualRelation = caseRecord.getDescription().getRelation("has-Description.Case_1");
                    individual = Framework.ontologies.getInstanceByLocalName(elements[1]);
                    if(individual != null){
                        concept = OntologyUtils.findOwner("Case_1", individual.getLocalName());
                        if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
                    }

                    individualRelation = caseRecord.getDescription().getRelation("has-Description.Case_2");
                    individual = Framework.ontologies.getInstanceByLocalName(elements[2]);
                    if(individual != null){
                        concept = OntologyUtils.findOwner("Case_2", individual.getLocalName());
                        if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
                    }

                    individualRelation = caseRecord.getDescription().getRelation("has-Description.Location");
                    individualRelation.getTarget().setValue(new Double(elements[3]));

                    individualRelation = caseRecord.getDescription().getRelation("has-Description.Participants");
                    individualRelation.getTarget().setValue(new Double(elements[4]));

                    individualRelation = caseRecord.getSolution().getRelation("has-Solution.Solution");
                    individual = Framework.ontologies.getInstanceByLocalName(elements[5]);
                    if(individual != null){
                        concept = OntologyUtils.findOwner("Solution", individual.getLocalName());
                        if(concept != null) individualRelation.getTarget().setValue(new ConceptType(concept, individual));
                    }

                    individualRelation = caseRecord.getSolution().getRelation("has-Solution.Property");
                    if(!elements[4].trim().equalsIgnoreCase("null")) individualRelation.getTarget().setValue(elements[6]);

                    cases.add(caseRecord);
                }catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }finally{
            try {
                reader.close();
            } catch (Exception ex) {}
        }
        return cases;
    }

    /**
     * retrieveSomeCases
     *
     * @param string String
     * @return Collection
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public Collection retrieveSomeCases(String string) {
        return retrieveAllCases();
    }


    /**
     * storeCases
     *
     * @param collection Collection
     * @todo Implement this jcolibri.cbrcase.Connector method
     */
    public void storeCases(Collection collection) {
        if(collection.size() == 0) return;
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter(databaseFile, true));
            writer.newLine();
            String header = "#--- Cases written by system at " + SimpleDateFormat.getDateTimeInstance().format(new Date()) + " --------------------------#";
            writer.write(header);

            Iterator cases = collection.iterator();

            while(cases.hasNext()){
                try {
                    String line = (++caseNumber) + ";";
                    CBRCase cbrCase = (CBRCase)cases.next();

                    IndividualRelation individualRelation = null;
                    jcolibri.cbrcase.Individual individual = null;

                    individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_1");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "null;" : ((ConceptType)individual.getValue()).getOntIndividual().getLocalName() + ";";

                    individualRelation = cbrCase.getDescription().getRelation("has-Description.Case_2");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "null;" : ((ConceptType)individual.getValue()).getOntIndividual().getLocalName() + ";";

                    individualRelation = cbrCase.getDescription().getRelation("has-Description.Location");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "1.0;" : ((Double)individual.getValue()).doubleValue() + ";";

                    individualRelation = cbrCase.getDescription().getRelation("has-Description.Participants");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "1.0;" : ((Double)individual.getValue()).doubleValue() + ";";

                    individualRelation = cbrCase.getSolution().getRelation("has-Solution.Solution");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "null;" : ((ConceptType)individual.getValue()).getOntIndividual().getLocalName() + ";";

                    individualRelation = cbrCase.getSolution().getRelation("has-Solution.Property");
                    individual = individualRelation.getTarget();
                    line += individual.getValue() == null ? "null" : individual.getValue();

                    writer.newLine();
                    writer.write(line);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }finally{
            try {
                writer.newLine();
                String footer = "#------------------------------------------------------------------------------#";
                writer.write(footer);
                writer.newLine();
                writer.flush();
                writer.close();
            } catch (Exception ex) {}
        }
    }

}
