package edu.ntnu.cikmr.arbitration.messages;

import java.io.ByteArrayInputStream;

import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.*;

public class ArbitrationMessage{
    private String peerName;
    private String peerID;
    private ConflictMessage conflict;
    private SolutionMessage solution;

    public String toXML(){
        String xml = "<arbitration peerName=\"" + peerName + "\" peerID=\"" + peerID + "\">\n";
        if(conflict != null) xml += conflict.toXML() + "\n";
        if(solution != null) xml += solution.toXML() + "\n";
        xml += "</arbitration>";
        return xml;
    }

    public static ArbitrationMessage fromXML(Node n){
        ArbitrationMessage a = new ArbitrationMessage();

        NamedNodeMap nodeMap = n.getAttributes();
        Node peerName = nodeMap.getNamedItem("peerName");
        a.setPeerName(peerName == null ? null : peerName.getNodeValue());
        Node peerID = nodeMap.getNamedItem("peerID");
        a.setPeerID(peerID == null ? null : peerID.getNodeValue());

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            if(childNode.getNodeName().equalsIgnoreCase("conflict")){
                ConflictMessage c = ConflictMessage.fromXML(childNode);
                a.setConflict(c);
            }else if(childNode.getNodeName().equalsIgnoreCase("solution")){
                SolutionMessage s = SolutionMessage.fromXML(childNode);
                a.setSolution(s);
            }
        }

        return a;
    }

    public static ArbitrationMessage fromXML(String xml){
        try{
            ByteArrayInputStream inputStream = new ByteArrayInputStream(xml.getBytes());
            Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(inputStream);
            NodeList nodeList = document.getElementsByTagName("arbitration");
            if(nodeList.getLength() == 1){
                ArbitrationMessage a = ArbitrationMessage.fromXML(nodeList.item(0));
                return a;
            }else{
                return null;
            }
        } catch(Exception ex){
            ex.printStackTrace();
        }
        return null;
    }

    public String getPeerID(){
        return peerID;
    }

    public String getPeerName(){
        return peerName;
    }

    public ConflictMessage getConflict(){
        return conflict;
    }

    public SolutionMessage getSolution(){
        return solution;
    }

    public void setPeerName(String peerName){
        this.peerName = peerName;
    }

    public void setPeerID(String peerID){
        this.peerID = peerID;
    }

    public void setSolution(SolutionMessage solution){
        this.solution = solution;
    }

    public void setConflict(ConflictMessage conflict){
        this.conflict = conflict;
    }

}
