package edu.ntnu.cikmr.arbitration.workflow;

import java.util.*;

import edu.ntnu.cikmr.XMLRepresentation;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Document;
import java.io.ByteArrayInputStream;
import javax.xml.parsers.DocumentBuilderFactory;

public class PeerWorkflowContainer implements XMLRepresentation{
    private List<Activity> activities;
    private String name;
    private String id;

    public PeerWorkflowContainer(){
        activities = Collections.synchronizedList(new ArrayList<Activity>());
    }

    public void fromXML(String xml){
        try{
            ByteArrayInputStream inputStream = new ByteArrayInputStream(xml.getBytes());
            Document document = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(inputStream);
            NodeList nodeList = document.getElementsByTagName("workflows");
            if(nodeList.getLength() == 1){
                fromXML(nodeList.item(0));
            }else{
                throw new Exception("Invalid workflow xml.");
            }
        } catch(Exception ex){
            ex.printStackTrace();
            throw new IllegalArgumentException("Invalid workflow xml.");
        }
    }

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node id = nodeMap.getNamedItem("peerID");
        this.id = id == null ? null : id.getNodeValue();
        Node name = nodeMap.getNamedItem("peerName");
        this.name = name == null ? null : name.getNodeValue();

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            if(childNode.getNodeName().equalsIgnoreCase("goal")){
                Goal goal = new Goal();
                goal.fromXML(childNode);
                add(goal);
            }else if(childNode.getNodeName().equalsIgnoreCase("activity")){
                Activity activity = new Activity();
                activity.fromXML(childNode);
                add(activity);
            }
        }
    }

    public void add(Activity activity){
        activities.add(activity);
    }

    public Iterator<Activity> activityIterator(){
        return activities.iterator();
    }

    public String toXML(){
        String xml = "<workflows peerName=\"" + name + "\" peerID=\"" + id + "\"";
        if(activities.isEmpty()){
            xml += "/>";
        }else{
            xml += ">\n";
            Iterator<Activity> iterator = activities.iterator();
            while(iterator.hasNext()){
                xml += iterator.next().toXML() + "\n";
            }
            xml += "</workflows>";
        }
        return xml;
    }

    public String getId(){
        return id;
    }

    public String getName(){
        return name;
    }

    public void setId(String id){
        this.id = id;
    }

    public void setName(String name){
        this.name = name;
    }

}
