package edu.ntnu.cikmr.gui;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import edu.ntnu.cikmr.Framework;
import java.util.ArrayList;
import jcolibri.cbrcore.CBRTask;
import jcolibri.cbrcore.exception.*;
import edu.ntnu.cikmr.arbitration.cbr.Case;
import jcolibri.cbrcase.CBRQuery;
import jcolibri.cbrcase.IndividualRelation;
import java.util.Iterator;
import jcolibri.cbrcase.Individual;
import java.util.Collection;
import edu.ntnu.cikmr.arbitration.cbr.Conflict;

public class ArbitrationComponent extends JPanel{
    private JMenuBar menuBar;
    private JMenu menuCycles;
    private JMenuItem[] menuCyclesItems;
    private JMenu menuContext;
    private JMenuItem[] menuContextItems;
    private JMenu menuTasks;
    private JMenuItem[] menuTaskItems;
    private JMenu menuArbitration;
    private JMenuItem[] menuArbitrationItems;
    private JTextArea textArea;


    public ArbitrationComponent() {
        super(new BorderLayout());

        menuCyclesItems = new JMenuItem[3];

        menuCyclesItems[0] = new JMenuItem("Precycle");
        menuCyclesItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTasks(Framework.arbitration.getCBR().preCycleTaskList);
            }
        });
        menuCyclesItems[1] = new JMenuItem("Cycle");
        menuCyclesItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTasks(Framework.arbitration.getCBR().cycleTaskList);
            }
        });
        menuCyclesItems[2] = new JMenuItem("Postcycle");
        menuCyclesItems[2].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTasks(Framework.arbitration.getCBR().postCycleTaskList);
            }
        });

        menuCycles = new JMenu("Cycles");
        menuCycles.add(menuCyclesItems[0]);
        menuCycles.add(menuCyclesItems[1]);
        menuCycles.add(menuCyclesItems[2]);


        menuTaskItems = new JMenuItem[9];
        menuTaskItems[0] = new JMenuItem("Obtain cases");
        menuTaskItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().obtainCasesTask);
            }
        });
        menuTaskItems[1] = new JMenuItem("Obtain query");
        menuTaskItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().obtainQueryTask);
            }
        });
        menuTaskItems[2] = new JMenuItem("Select working cases");
        menuTaskItems[2].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().selectWorkingCasesTask);
            }
        });
        menuTaskItems[3] = new JMenuItem("Compute similarity");
        menuTaskItems[3].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().computeSimilarityTask);
            }
        });
        menuTaskItems[4] = new JMenuItem("Select best cases");
        menuTaskItems[4].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().selectBestCasesTask);
            }
        });
        menuTaskItems[5] = new JMenuItem("Prepare cases for adaption");
        menuTaskItems[5].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().prepareCasesForAdaptionTask);
            }
        });
        menuTaskItems[6] = new JMenuItem("Atomic reuse");
        menuTaskItems[6].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().atomicReuseTask);
            }
        });
        menuTaskItems[7] = new JMenuItem("Store cases");
        menuTaskItems[7].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().storeCasesTask);
            }
        });

        menuTaskItems[8] = new JMenuItem("Close connector");
        menuTaskItems[8].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                executeTask(Framework.arbitration.getCBR().closeConnectorTask);
            }
        });

        menuTasks = new JMenu("Tasks");
        menuTasks.add(menuTaskItems[0]);
        menuTasks.addSeparator();
        for (int i = 1; i < 8; i++) menuTasks.add(menuTaskItems[i]);
        menuTasks.add(menuTaskItems[8]);


        menuContextItems = new JMenuItem[2];
        menuContextItems[0] = new JMenuItem("Show");
        menuContextItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                getContext();
            }
        });
        menuContextItems[1] = new JMenuItem("Reset");
        menuContextItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                resetContext();
            }
        });

        menuContext = new JMenu("Context");
        menuContext.add(menuContextItems[0]);
        menuContext.add(menuContextItems[1]);

        menuArbitrationItems = new JMenuItem[2];
        menuArbitrationItems[0] = new JMenuItem("Create query");
        menuArbitrationItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                createQuery();
            }
        });

        menuArbitrationItems[1] = new JMenuItem("Check for conflicts");
        menuArbitrationItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                checkForConflicts();
            }
        });

        menuArbitration = new JMenu("Arbitration");
        menuArbitration.add(menuArbitrationItems[0]);
        menuArbitration.add(menuArbitrationItems[1]);

        menuBar = new JMenuBar();
        menuBar.add(menuCycles);
        menuBar.add(menuTasks);
        menuBar.add(menuContext);
        menuBar.add(menuArbitration);

        textArea = new JTextArea();
        JScrollPane textAreaScroll = new JScrollPane(textArea);

        add(menuBar, BorderLayout.NORTH);
        add(textAreaScroll, BorderLayout.CENTER);
    }

    private void executeTasks(ArrayList<CBRTask> taskList){
        try {
            Framework.arbitration.getCBR().executeTasks(taskList);
            getContext();
        } catch (ExecutionException ex) {
            textArea.setText("Unable to execute task(s).");
        }
    }

    private void executeTask(CBRTask task){
        try {
            Framework.arbitration.getCBR().executeTask(task);
            getContext();
        } catch (ExecutionException ex) {
            textArea.setText("Unable to execute task.");
        }
    }

    private void getContext(){
        String context = Framework.arbitration.getCBR().getContext().toString();
        textArea.setText(context);
    }

    private void resetContext(){
        Framework.arbitration.getCBR().reset();
    }

    private Case c;

    private void createQuery(){
        String case1 = JOptionPane.showInputDialog(this, "Enter case 1:", "Create query", JOptionPane.INFORMATION_MESSAGE);
        if(case1 == null) return;
        String case2 = JOptionPane.showInputDialog(this, "Enter case 2:", "Create query", JOptionPane.INFORMATION_MESSAGE);
        if(case2 == null) return;

        c = new Case(null, case1, case2, 0, 0, null, null);
    }

    private void checkForConflicts(){
        Conflict[] conflicts = Framework.arbitration.getCBR().checkForConflict(c);
        if(conflicts == null){
            System.out.println("No conflicts found.");
        }else{
            System.out.println(conflicts.length + " possible conflict(s) found.");
        }
    }

}
