package edu.ntnu.cikmr.gui;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.util.Iterator;

import javax.swing.*;

import edu.ntnu.cikmr.*;
import edu.ntnu.cikmr.services.location.*;
import edu.ntnu.cikmr.services.resources.*;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class LocationComponent extends JPanel implements ResourceListener{
    private ImageIcon rooms;
    private ImageIcon peerImage;

    private LocationSensorResource locationSensorResource;

    private java.util.List<LocationSensorResourceThread> locationSensorResources;

    private int roomsPadding = 3;

    public LocationComponent(){
        setLayout(null);
        rooms = new ImageIcon("edu/ntnu/cikmr/gui/graphics/rooms.gif");
        setMinimumSize(new Dimension(425, 306));
        setPreferredSize(new Dimension(425, 306));
        setOpaque(true);
        setBackground(Color.WHITE);

        peerImage = new ImageIcon("edu/ntnu/cikmr/gui/graphics/peer.gif");

        locationSensorResources = Collections.synchronizedList(new ArrayList<LocationSensorResourceThread>());

        locationSensorResource = (LocationSensorResource)Framework.network.getServiceManager().get(LocationSensorResource.class);
        if(locationSensorResource == null){
            locationSensorResource = new LocationSensorResource();
            Framework.network.getServiceManager().add(locationSensorResource);
        }
        locationSensorResource.setLocation("White_room");
        locationSensorResource.setLocationX(140);
        locationSensorResource.setLocationY(160);
        locationSensorResource.setLocationZ(0);

        addAllLocationSensorResources();
        Framework.resources.addResourceListener(this);

        addMouseListener(new MouseAdapter(){
            public void mouseClicked(MouseEvent e){ moveMe(e); }
        });
        new RepaintThread();
    }

    private void addAllLocationSensorResources(){
        Resource[] resources = Framework.resources.getAll();
        for (int i = 0; i < resources.length; i++) {
            Resource resource = resources[i];
            if(resource.getResourceDescription().getType().getLocalName().equalsIgnoreCase("Location_sensor")){
                LocationSensorResourceThread thread = new LocationSensorResourceThread(resource);
                Framework.resources.addResourceListener(thread);
                locationSensorResources.add(thread);
            }
        }
    }

    public void paintComponent(Graphics g){
        super.paintComponent(g);
        g.drawImage(rooms.getImage(), roomsPadding, roomsPadding, this);

        try{
            Iterator<LocationSensorResourceThread> i = locationSensorResources.iterator();
            while(i.hasNext()) drawPeer(g, i.next());
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }

    private void drawPeer(Graphics g, LocationSensorResourceThread r){

        int x = r.x - (peerImage.getIconWidth() / 2) + roomsPadding;
        int y = r.y - (peerImage.getIconHeight() / 2) + roomsPadding;
        g.drawImage(peerImage.getImage(), x, y, this);

        String room = r.location.replace('_', ' ');

        String text = r.resource.getPeerName() + " (" + room + ")";

        Rectangle2D r2d = g.getFontMetrics().getStringBounds(text, g);
        x = r.x - (int)r2d.getCenterX();
        y += peerImage.getIconHeight() + r2d.getHeight();
        g.drawString(text, x, y);
    }

    private void moveMe(MouseEvent e){
        int x = e.getX() - roomsPadding;
        int y = e.getY() - roomsPadding;
        String llocation = "";
        if(x >= 5 && x <= 5+88 && y >= 5 && y <= 5+184){
            // Pink room
            llocation = "Pink_room";
        }else if(x >= 98 && x <= 98+316 && y >= 5 && y <= 5+112){
            // Yellow room
            llocation = "Yellow_room";
        }else if(x >= 98 && x <= 98+86 && y >= 122 && y <= 122+67){
            // White room
            llocation = "White_room";
        }else if(x >= 189 && x <= 189+225 && y >= 122 && y <= 122+173){
            // Green room
            llocation = "Green_room";
        }else if(x >= 5 && x <= 5+179 && y >= 194 && y <= 194+101){
            // Blue room
            llocation = "Blue_room";
        }
        locationSensorResource.setLocation(llocation);
        locationSensorResource.setLocationX(x);
        locationSensorResource.setLocationY(y);
        locationSensorResource.setLocationZ(0);
    }

    public void resourceFound(ResourceEvent e) {
        try {
            if(e.getResource().getResourceDescription().getType().getLocalName().equalsIgnoreCase("Location_sensor")){
                LocationSensorResourceThread thread = new LocationSensorResourceThread(e.getResource());
                Framework.resources.addResourceListener(thread);
                locationSensorResources.add(thread);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void resourceLost(ResourceEvent e){}

    private class LocationSensorResourceThread extends Thread implements ResourceListener{
        private boolean run;
        private int delay;
        private int x;
        private int y;
        private int z;
        private String location;
        private Resource resource;

        public LocationSensorResourceThread(Resource resource){
            run = true;
            delay = 10000;
            this.resource = resource;

            x = 0;
            y = 0;
            z = 0;
            location = "";

            start();
        }

        public void run(){
            while(run){

                try {

                    ResourceMethod[] methods = new ResourceMethod[4];
                    methods[0] = resource.getResourceDescription().getMethod("getLocationX");
                    methods[1] = resource.getResourceDescription().getMethod("getLocationY");
                    methods[2] = resource.getResourceDescription().getMethod("getLocationZ");
                    methods[3] = resource.getResourceDescription().getMethod("getLocation");

                    Object[][] parameters = new Object[4][0];

                    InvocationResult[] result = resource.invokeMethods(methods, parameters);

                    x = Integer.parseInt(result[0].getReturnValue().toString(), 10);
                    y = Integer.parseInt(result[1].getReturnValue().toString(), 10);
                    z = Integer.parseInt(result[2].getReturnValue().toString(), 10);
                    location = result[3].getReturnValue().toString();

                } catch (Exception ex) {}

                /**
                try {
                    InvocationResult lx = resource.getResourceDescription().getMethod("getLocationX").invokeMethod(null);
                    x = Integer.parseInt(lx.getReturnValue().toString(), 10);
                } catch (Exception ex) {}
                try {
                    InvocationResult ly = resource.getResourceDescription().getMethod("getLocationY").invokeMethod(null);
                    y = Integer.parseInt(ly.getReturnValue().toString(), 10);
                } catch (Exception ex) {}
                try {
                    InvocationResult lz = resource.getResourceDescription().getMethod("getLocationZ").invokeMethod(null);
                    z = Integer.parseInt(lz.getReturnValue().toString(), 10);
                } catch (Exception ex) {}
                try {
                    InvocationResult llocation = resource.getResourceDescription().getMethod("getLocation").invokeMethod(null);
                    location = llocation.getReturnValue().toString();
                } catch (Exception ex) {}
                **/

                try {
                    Thread.sleep(delay);
                } catch (Exception ex) {}
            }
        }

        public void resourceFound(ResourceEvent e) {}
        public void resourceLost(ResourceEvent e) {
            if(e.getResource() == resource){
                run = false;
                try {
                    this.interrupt();
                    this.join();
                } catch (Exception ex) {}
                locationSensorResources.remove(this);
            }
        }

    }

    private class RepaintThread extends Thread{
        private boolean run;
        private int delay;

        public RepaintThread(){
            run = true;
            delay = 5000;
            start();
        }

        public void run(){
            while(run){
                try {
                    if(isShowing()) repaint();
                    Thread.sleep(delay);
                } catch (Exception ex) {}
            }
        }

    }

}
