package edu.ntnu.cikmr.gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.Collections;

import javax.swing.*;

import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.workflowenactment.Workflow;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.event.*;

public class WorkflowDefinitions extends JPanel{
    private JTextArea workflowXML;
    private JList definitions;
    private DefinitionsModel definitionsModel;
    private JMenuBar menuBar;
    private JMenu menuDefinitions;
    private JMenuItem menuDefinitionsItems[];
    private JMenu menuWorkflow;
    private JMenuItem menuWorkflowItems[];

    public WorkflowDefinitions(){
        super(new BorderLayout());

        menuDefinitionsItems = new JMenuItem[3];
        menuDefinitionsItems[0] = new JMenuItem("Load from file...");
        menuDefinitionsItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                loadFromFile();
            }
        });
        menuDefinitionsItems[1] = new JMenuItem("Create from textfield");
        menuDefinitionsItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                createFromTextfield();
            }
        });
        menuDefinitionsItems[2] = new JMenuItem("Remove selected");
        menuDefinitionsItems[2].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                removeSelectedDefinition();
            }
        });

        menuDefinitions = new JMenu("Definitions");
        menuDefinitions.add(menuDefinitionsItems[0]);
        menuDefinitions.add(menuDefinitionsItems[1]);
        menuDefinitions.addSeparator();
        menuDefinitions.add(menuDefinitionsItems[2]);

        menuWorkflowItems = new JMenuItem[3];
        menuWorkflowItems[0] = new JMenuItem("Instanciate selected definition");
        menuWorkflowItems[0].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                instanciateFromList();
            }
        });
        menuWorkflowItems[1] = new JMenuItem("Instanciate from textfield");
        menuWorkflowItems[1].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                instanciateFromTextfield();
            }
        });
        menuWorkflowItems[2] = new JMenuItem("Clear workflow XML");
        menuWorkflowItems[2].addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                clearWorkflowXML();
            }
        });


        menuWorkflow = new JMenu("Workflows");
        menuWorkflow.add(menuWorkflowItems[0]);
        menuWorkflow.add(menuWorkflowItems[1]);
        menuWorkflow.addSeparator();
        menuWorkflow.add(menuWorkflowItems[2]);

        menuBar = new JMenuBar();
        menuBar.add(menuDefinitions);
        menuBar.add(menuWorkflow);

        definitionsModel = new DefinitionsModel();
        definitions = new JList(definitionsModel);
        definitions.setCellRenderer(new DefinitionListCellRenderer());
        definitions.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        definitions.addMouseListener(new MouseAdapter(){
                        public void mouseClicked(MouseEvent e){
                            if(e.getClickCount() == 2) showDefinitionXML();
                        }
                    });

        JScrollPane definitionsScroll = new JScrollPane(definitions);
        definitionsScroll.setPreferredSize(new Dimension(150, 100));
        definitionsScroll.setMinimumSize(new Dimension(150, 40));
        definitionsScroll.setMaximumSize(new Dimension(150, 10000));

        workflowXML = new JTextArea();
        clearWorkflowXML();
        JScrollPane workflowScroll = new JScrollPane(workflowXML);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, definitionsScroll, workflowScroll);
        splitPane.setBorder(null);

        this.add(menuBar, BorderLayout.NORTH);
        this.add(splitPane, BorderLayout.CENTER);
    }

    private void clearWorkflowXML(){
        workflowXML.setText("<goal id=\"\" type=\"\">\n\n<incentive interval=\"1000\" iftrue=\"\" iffalse=\"\" binding=\"and\">\n\n</incentive>\n\n</goal>");
    }

    private void instanciateFromTextfield(){
        String xml = workflowXML.getText();
        Workflow workflow = Framework.scheduler.addWorkflow(xml);
        if(workflow == null) JOptionPane.showMessageDialog(this, "Unable to instanciate workflow.", "Instanciate", JOptionPane.ERROR_MESSAGE);
    }

    private void instanciateFromList(){
        Object o = definitions.getSelectedValue();
        if(o instanceof Definition){
            String xml = ((Definition)o).xml;
            Workflow workflow = Framework.scheduler.addWorkflow(xml);
            if(workflow == null) JOptionPane.showMessageDialog(this, "Unable to instanciate workflow.", "Instanciate", JOptionPane.ERROR_MESSAGE);
        }
    }

    private void loadFromFile(){
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        fileChooser.setMultiSelectionEnabled(false);
        int i = fileChooser.showOpenDialog(this);
        if(i == JFileChooser.APPROVE_OPTION){
            File file = fileChooser.getSelectedFile();
            try{
                FileInputStream inputStream = new FileInputStream(file);
                byte[] buffer = new byte[(int)file.length()];
                inputStream.read(buffer);
                definitionsModel.add(new String(buffer));
            } catch(Exception ex){
                JOptionPane.showMessageDialog(this, "Unable to load workflow definition file.", "Load", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    private void createFromTextfield(){
        definitionsModel.add(workflowXML.getText());
    }

    private void removeSelectedDefinition(){
        try{
            definitionsModel.remove(definitions.getSelectedIndex());
        } catch(Exception ex){
            JOptionPane.showMessageDialog(this, "Unable to remove workflow definition.", "Remove definition", JOptionPane.ERROR_MESSAGE);
        }
    }

    private void showDefinitionXML(){
        Object o = definitions.getSelectedValue();
        if(o instanceof Definition) workflowXML.setText(((Definition)o).xml);
    }


    private class Definition{
        private String name;
        private String xml;
    }

    private class DefinitionsModel extends AbstractListModel{
        private java.util.List<Definition> definitions;

        public DefinitionsModel(){
            definitions = Collections.synchronizedList(new ArrayList<Definition>());
        }

        public void add(String xml){
            String name = (String)JOptionPane.showInputDialog(WorkflowDefinitions.this, "Enter a name for the definition.", "New definition", JOptionPane.INFORMATION_MESSAGE);
            if(name == null) return;
            Definition d = new Definition();
            d.xml = xml;
            d.name = name;
            definitions.add(d);
            this.fireIntervalAdded(WorkflowDefinitions.this, definitions.size(), definitions.size());
        }

        public void remove(int i){
            definitions.remove(i);
            this.fireIntervalAdded(WorkflowDefinitions.this, i, i);
        }

        public int getSize(){
            return definitions.size();
        }

        public Object getElementAt(int _int){
            return definitions.get(_int);
        }

    }

    private class DefinitionListCellRenderer extends JLabel implements ListCellRenderer{

        public DefinitionListCellRenderer(){
            setOpaque(true);
        }

        public Component getListCellRendererComponent(JList jList, Object object, int _int, boolean _boolean, boolean _boolean4){
            setBackground(_boolean ? jList.getSelectionBackground() : jList.getBackground());
            setForeground(_boolean ? jList.getSelectionForeground() : jList.getForeground());
            setText(object instanceof Definition ? ((Definition)object).name : object.toString());
            return this;
        }

    }

}
