package edu.ntnu.cikmr.gui;

import javax.swing.*;
import edu.ntnu.cikmr.workflowenactment.*;
import edu.ntnu.cikmr.Framework;
import java.awt.*;
import javax.swing.table.AbstractTableModel;
import java.awt.event.*;
import java.util.EventObject;
import edu.ntnu.cikmr.log.Log;

public class WorkflowPanel extends JPanel implements WorkflowLifecycleListener{
    private Workflow workflow;
    private JPanel localPanel;
    private JButton removeButton;
    private JButton startButton;
    private JButton haltButton;
    private JButton addDefinedButton;
    private JButton pipeIDButton;
    private JButton remoteConnectButton;
    private JButton addParticipantButton;
    private JButton removeParticipantButton;
    private JList participantsList;
    private JTable logTable;
    private WorkflowTableModel workflowTableModel;
    private ParticipantsListModel participantsListModel;

    private JPanel remotePanel;
    private JLabel runningLabel;
    private JButton leaveButton;
    private JButton remoteRemoveButton;
    private JList remoteParticipantsList;

    public WorkflowPanel(Workflow workflow){
        super(new BorderLayout());
        this.workflow = workflow;

        //----------------------------------------------------------------------
        addDefinedButton = new JButton("Set variable");
        addDefinedButton.setPreferredSize(new Dimension(100, 20));
        addDefinedButton.setMinimumSize(new Dimension(100, 20));
        addDefinedButton.setMaximumSize(new Dimension(100, 20));
        addDefinedButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 addDefined();
                             }
                         });

        startButton = new JButton("Start");
        startButton.setPreferredSize(new Dimension(100, 20));
        startButton.setMinimumSize(new Dimension(100, 20));
        startButton.setMaximumSize(new Dimension(100, 20));
        startButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 start();
                             }
                         });
        startButton.setEnabled(!workflow.isExecuting());

        haltButton = new JButton("Halt");
        haltButton.setPreferredSize(new Dimension(100, 20));
        haltButton.setMinimumSize(new Dimension(100, 20));
        haltButton.setMaximumSize(new Dimension(100, 20));
        haltButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 halt();
                             }
                         });
        haltButton.setEnabled(workflow.isExecuting());

        removeButton = new JButton("Remove");
        removeButton.setPreferredSize(new Dimension(100, 20));
        removeButton.setMinimumSize(new Dimension(100, 20));
        removeButton.setMaximumSize(new Dimension(100, 20));
        removeButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 remove();
                             }
                         });

        pipeIDButton = new JButton("Pipe ID");
        pipeIDButton.setPreferredSize(new Dimension(100, 20));
        pipeIDButton.setMinimumSize(new Dimension(100, 20));
        pipeIDButton.setMaximumSize(new Dimension(100, 20));
        pipeIDButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 showPipeID();
                             }
                         });

        remoteConnectButton = new JButton("Connect");
        remoteConnectButton.setPreferredSize(new Dimension(100, 20));
        remoteConnectButton.setMinimumSize(new Dimension(100, 20));
        remoteConnectButton.setMaximumSize(new Dimension(100, 20));
        remoteConnectButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 remoteConnect();
                             }
                         });

        JLabel participantsLabel = new JLabel("Participants");
        participantsLabel.setPreferredSize(new Dimension(100, 20));
        participantsLabel.setMinimumSize(new Dimension(100, 20));
        participantsLabel.setMaximumSize(new Dimension(100, 20));

        addParticipantButton = new JButton("Add");
        addParticipantButton.setPreferredSize(new Dimension(100, 20));
        addParticipantButton.setMinimumSize(new Dimension(100, 20));
        addParticipantButton.setMaximumSize(new Dimension(100, 20));
        addParticipantButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 addParticipant();
                             }
                         });

        removeParticipantButton = new JButton("Remove");
        removeParticipantButton.setPreferredSize(new Dimension(100, 20));
        removeParticipantButton.setMinimumSize(new Dimension(100, 20));
        removeParticipantButton.setMaximumSize(new Dimension(100, 20));
        removeParticipantButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 removeParticipant();
                             }
                         });

        participantsListModel = new ParticipantsListModel();
        participantsList = new JList(participantsListModel);
        JScrollPane participantsScroll = new JScrollPane(participantsList);
        participantsScroll.setPreferredSize(new Dimension(100, 200));
        participantsScroll.setMinimumSize(new Dimension(100, 50));
        participantsScroll.setMaximumSize(new Dimension(100, 10000));

        localPanel = new JPanel(new FlowLayout());
        localPanel.setPreferredSize(new Dimension(120, 100));
        localPanel.setMinimumSize(new Dimension(120, 100));
        localPanel.setMaximumSize(new Dimension(120, 10000));
        localPanel.add(addDefinedButton);
        localPanel.add(startButton);
        localPanel.add(haltButton);
        localPanel.add(removeButton);
        localPanel.add(pipeIDButton);
        localPanel.add(remoteConnectButton);
        localPanel.add(participantsLabel);
        localPanel.add(addParticipantButton);
        localPanel.add(removeParticipantButton);
        localPanel.add(participantsScroll);
        //----------------------------------------------------------------------
        runningLabel = new JLabel(workflow.isExecuting() ? "Running..." : "Stopped");
        runningLabel.setPreferredSize(new Dimension(100, 20));
        runningLabel.setMinimumSize(new Dimension(100, 20));
        runningLabel.setMaximumSize(new Dimension(100, 20));

        leaveButton = new JButton("Leave");
        leaveButton.setPreferredSize(new Dimension(100, 20));
        leaveButton.setMinimumSize(new Dimension(100, 20));
        leaveButton.setMaximumSize(new Dimension(100, 20));
        leaveButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 leave();
                             }
                         });

        remoteRemoveButton = new JButton("Remove");
        remoteRemoveButton.setPreferredSize(new Dimension(100, 20));
        remoteRemoveButton.setMinimumSize(new Dimension(100, 20));
        remoteRemoveButton.setMaximumSize(new Dimension(100, 20));
        remoteRemoveButton.addActionListener(new ActionListener(){
                             public void actionPerformed(ActionEvent e){
                                 remove();
                             }
                         });

        JLabel remoteParticipantsLabel = new JLabel("Participants");
        remoteParticipantsLabel.setPreferredSize(new Dimension(100, 20));
        remoteParticipantsLabel.setMinimumSize(new Dimension(100, 20));
        remoteParticipantsLabel.setMaximumSize(new Dimension(100, 20));

        remoteParticipantsList = new JList(participantsListModel);
        JScrollPane remoteParticipantsScroll = new JScrollPane(remoteParticipantsList);
        remoteParticipantsScroll.setPreferredSize(new Dimension(100, 200));
        remoteParticipantsScroll.setMinimumSize(new Dimension(100, 50));
        remoteParticipantsScroll.setMaximumSize(new Dimension(100, 10000));

        remotePanel = new JPanel(new FlowLayout());
        remotePanel.setPreferredSize(new Dimension(120, 100));
        remotePanel.setMinimumSize(new Dimension(120, 100));
        remotePanel.setMaximumSize(new Dimension(120, 10000));
        remotePanel.add(runningLabel);
        remotePanel.add(leaveButton);
        remotePanel.add(remoteRemoveButton);
        remotePanel.add(remoteParticipantsLabel);
        remotePanel.add(remoteParticipantsScroll);
        //----------------------------------------------------------------------

        workflowTableModel = new WorkflowTableModel();
        logTable = new JTable(workflowTableModel);
        JScrollPane logScroll = new JScrollPane(logTable);

        add(workflow.isExecutedLocally() ? localPanel : remotePanel, BorderLayout.WEST);
        add(logScroll, BorderLayout.CENTER);

        workflow.addWorkflowLifecycleListener(this);
    }

    public void addParticipant(){
        String peerID = JOptionPane.showInputDialog(this, "Enter peer ID", "New participant", JOptionPane.INFORMATION_MESSAGE);
        if(peerID == null) return;
        String peerName = JOptionPane.showInputDialog(this, "Enter peer name", "New participant", JOptionPane.INFORMATION_MESSAGE);
        if(peerName == null) return;
        workflow.addParticipant(peerID, peerName);
    }

    private void removeParticipant(){
        int index = participantsList.getSelectedIndex();
        if(index >= 0){
            Participant p = workflow.getParticipant(index);
            if(p != null) workflow.removeParticipant(p.getPeerID());
        }
    }

    private void leave(){
        workflow.moveLocal();
    }

    private void showPipeID(){
        String pipeIDMessage = workflow.getServerPipeID();
        if(pipeIDMessage == null) pipeIDMessage = "No server pipe found.";
        JOptionPane.showMessageDialog(this, new JTextField(pipeIDMessage), "Pipe ID", JOptionPane.INFORMATION_MESSAGE);
    }

    public void remoteConnect(){
        String pipeID = JOptionPane.showInputDialog(this, "Enter pipe ID", "Connect", JOptionPane.INFORMATION_MESSAGE);
        if(pipeID != null) workflow.remoteConnect(pipeID);
    }

    public Workflow getWorkflow(){
        return workflow;
    }

    private void updateStatus(){
        startButton.setEnabled(!workflow.isExecuting());
        haltButton.setEnabled(workflow.isExecuting());
        runningLabel.setText(workflow.isExecuting() ? "Running..." : "Stopped");
    }

    private void addDefined(){
        String name = JOptionPane.showInputDialog(this, "Enter variable name.", "Set variable", JOptionPane.INFORMATION_MESSAGE);
        if(name == null) return;
        String value = JOptionPane.showInputDialog(this, "Enter variable value.", "Set variable", JOptionPane.INFORMATION_MESSAGE);
        if(value == null) return;
        workflow.setDefined(name, value);
    }

    private void start(){
        workflow.execute();
    }

    private void halt(){
        workflow.halt();
    }

    private void remove(){
        Framework.scheduler.removeWorkflow(workflow);
    }

    public void workflowStarted(WorkflowEvent e){
        updateStatus();
    }

    public void workflowStopped(WorkflowEvent e){
        updateStatus();
    }

    public void workflowFinished(WorkflowEvent e){
        updateStatus();
    }

    public void workflowExecutionMoved(WorkflowEvent e){
        remove(localPanel);
        remove(remotePanel);
        add(workflow.isExecutedLocally() ? localPanel : remotePanel, BorderLayout.WEST);
        updateStatus();
        repaint();
    }

    public void workflowParticipantsChanged(WorkflowEvent e){
        participantsListModel.fireUpdated();
    }

    private class WorkflowTableModel extends AbstractTableModel implements WorkflowLogListener{

        public WorkflowTableModel(){
            workflow.getLog().addWorkflowLogListener(this);
        }

        public int getRowCount(){
            return workflow.getLog().getEntriesCount();
        }

        public int getColumnCount(){
            return 4;
        }

        public Object getValueAt(int row, int column){
            WorkflowLogEntry entry = workflow.getLog().getEntry(row);
            switch(column){
                case 0: return entry.getActivity() == null ? "" : entry.getActivity();
                case 1: return entry.getStatement() == null ? "" : entry.getStatement();
                case 2:
                    switch(entry.getLevel()){
                        case Log.LEVEL_DEBUG: return "Debug";
                        case Log.LEVEL_ERROR: return "Error";
                        case Log.LEVEL_INFORMATIVE: return "Informative";
                        case Log.LEVEL_SEVERE: return "Severe";
                    }
                    return "-";
                case 3: return entry.getMessage() == null ? "" : entry.getMessage();
            }
            return "-";
        }

        public void logUpdated(EventObject e){
            this.fireTableDataChanged();
        }

        public String getColumnName(int index){
            switch(index){
                case 0: return "Activity";
                case 1: return "Statement";
                case 2: return "Level";
                case 3: return "Message";
            }
            return "-";
        }

    }

    private class ParticipantsListModel extends AbstractListModel{

        public int getSize(){
            return workflow.getParticipantsCount();
        }

        public Object getElementAt(int _int){
            return workflow.getParticipant(_int).getPeerName();
        }

        private void fireUpdated(){
            this.fireContentsChanged(this, 0, getSize());
        }

    }

}
