package edu.ntnu.cikmr.network;

import javax.swing.event.*;

public class ConnectionStatus {
    public static final int STATUS_DISCONNECTED = 0;
    public static final int STATUS_CONFIGURATING_NETWORK = 1;
    public static final int STATUS_CONNECTING_TO_NET_PEER_GROUP = 2;
    public static final int STATUS_CONNECTING_TO_PEER_GROUP = 3;
    public static final int STATUS_DISCONNECTING = 4;
    public static final int STATUS_CONNECTED = 5;
    //--------------------------------------------------------------------------
    public static final int ERROR_NONE = 0;
    public static final int ERROR_UNKNOWN = 1;
    public static final int ERROR_TIMED_OUT = 2;
    public static final int ERROR_CONNECTING_TO_NET_PEER_GROUP = 3;
    public static final int ERROR_CONNECTING_TO_PEER_GROUP = 4;
    //--------------------------------------------------------------------------
    private int status;
    private int errorCode;
    private String errorDescription;
    private String peerName;
    //--------------------------------------------------------------------------
    private EventListenerList eventListenerList;

    private boolean configured;
    private boolean connectedToNetPeerGroup;



    public ConnectionStatus(){
        eventListenerList = new EventListenerList();
        configured = false;
        connectedToNetPeerGroup = false;
        reset();
    }

    protected void reset(){
        status = STATUS_DISCONNECTED;
        errorCode = ERROR_NONE;
        errorDescription = "";
    }

    public boolean isConnected() {
        return status == STATUS_CONNECTED;
    }

    public boolean isConnectionInProgress() {
        return (status != STATUS_DISCONNECTED) && (status != STATUS_CONNECTED);
    }

    public boolean hasError() {
        return errorCode != ERROR_NONE;
    }

    public int getErrorCode() {
        return errorCode;
    }

    public String getErrorDescription() {
        return errorDescription;
    }

    public int getStatus(){
        return status;
    }

    public String getPeerName(){
        return peerName;
    }

    protected boolean isConfigured(){
        return configured;
    }

    protected boolean isConnectedToNetPeerGroup(){
        return connectedToNetPeerGroup;
    }

    protected void setErrorDescription(String errorDescription) {
        this.errorDescription = errorDescription;
    }

    protected void setErrorCode(int errorCode) {
        this.errorCode = errorCode;
    }

    public void setStatus(int status){
        this.status = status;
    }

    protected void setPeerName(String peerName){
        this.peerName = peerName;
    }

    protected void setConfigured(boolean configured){
        this.configured = configured;
    }

    protected void setConnectedToNetPeerGroup(boolean connectedToNetPeerGroup){
        this.connectedToNetPeerGroup = connectedToNetPeerGroup;
    }

    public void addConnectionStatusListener(ConnectionStatusListener connectionStatusListener){
        eventListenerList.add(ConnectionStatusListener.class, connectionStatusListener);
    }

    public void removeConnectionStatusListener(ConnectionStatusListener connectionStatusListener){
        eventListenerList.remove(ConnectionStatusListener.class, connectionStatusListener);
    }

    protected void fireConnectionStatusUpdated(){
        ConnectionStatusListener[] listeners = (ConnectionStatusListener[])eventListenerList.getListeners(ConnectionStatusListener.class);
        for(int i = listeners.length - 1; i >= 0; i--) listeners[i].connectionStatusUpdated(this);
    }

}
