package edu.ntnu.cikmr.network;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;

import javax.security.cert.CertificateException;

import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.log.Log;
import net.jxta.discovery.DiscoveryService;
import net.jxta.document.Advertisement;
import net.jxta.exception.PeerGroupException;
import net.jxta.peergroup.*;
import net.jxta.platform.NetworkConfigurator;
import net.jxta.protocol.PeerGroupAdvertisement;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class PeerGroupConnection implements Runnable{
    private NetworkConfigurator networkConfigurator;
    private PeerGroup netPeerGroup;
    private PeerGroup peerGroup;
    private ConnectionStatus connectionStatus;
    private Thread thread;

    protected PeerGroupConnection(){
        connectionStatus = new ConnectionStatus();
    }

    protected synchronized void connect(){
        if((thread != null && thread.isAlive()) || connectionStatus.isConnected()) return;
        thread = new Thread(this);
        thread.start();
    }

    protected synchronized void disconnect() throws PeerGroupException{
        if(connectionStatus.isConnected()){
            connectionStatus.setStatus(ConnectionStatus.STATUS_DISCONNECTING);
            connectionStatus.fireConnectionStatusUpdated();
            peerGroup.stopApp();
            netPeerGroup.stopApp();
            connectionStatus.setStatus(ConnectionStatus.STATUS_DISCONNECTED);
            connectionStatus.fireConnectionStatusUpdated();

        }
    }

    public void run(){
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Connecting.");
        connectionStatus.reset();
        connectionStatus.fireConnectionStatusUpdated();
        configureNetwork();
        try{
            connectToNetPeerGroup();
        } catch(PeerGroupException pgEx){
            connectionStatus.setErrorCode(ConnectionStatus.ERROR_CONNECTING_TO_NET_PEER_GROUP);
            connectionStatus.setErrorDescription(pgEx.getMessage());
            connectionStatus.fireConnectionStatusUpdated();
            return;
        }

        try{
            connectToPeerGroup();
        } catch(PeerGroupException pgEx){
            connectionStatus.setErrorCode(ConnectionStatus.ERROR_CONNECTING_TO_PEER_GROUP);
            connectionStatus.setErrorDescription(pgEx.getMessage());
            connectionStatus.fireConnectionStatusUpdated();
            return;
        }

        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Connected.");
        connectionStatus.setStatus(ConnectionStatus.STATUS_CONNECTED);
        connectionStatus.fireConnectionStatusUpdated();
    }

    private void configureNetwork(){
        if(connectionStatus.isConfigured()) return;
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Configuring.");
        //PeerGroupID peerGroupID = net.jxta.id.IDFactory.newPeerGroupID(new String("NTNU:Norway - Master thesis 2007 - Christian Indahl & Kjell Martin Rud").getBytes());
        connectionStatus.setStatus(ConnectionStatus.STATUS_CONFIGURATING_NETWORK);
        connectionStatus.fireConnectionStatusUpdated();

        if(networkConfigurator == null) networkConfigurator = new NetworkConfigurator();

        if (!networkConfigurator.exists()) {
            createNewConfiguration();
        } else {
            try{
                loadExistingConfiguration();
            } catch(IOException ioEx){
                createNewConfiguration();
            } catch(CertificateException cEx){
                try{
                    loadExistingConfiguration();
                } catch(Exception ex){
                    createNewConfiguration();
                }
            }
        }
    }

    private void loadExistingConfiguration() throws IOException, CertificateException{
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Loading existing configuration.");
        File pc = new File(networkConfigurator.getHome(), "PlatformConfig");
        try {
            networkConfigurator.load(pc.toURI());
        }catch(IOException ioEx){
            Framework.log.logMessage(Framework.network, Log.LEVEL_ERROR, "Unable to load existing configuration due to I/O exception.");
            throw ioEx;
        }catch(CertificateException ce){
            Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Invalid root certificate. Creating new.");
            try {
                networkConfigurator.setPrincipal("master");
                networkConfigurator.setPassword("thesis");
                networkConfigurator.save();
                throw ce;
            } catch (IOException e) {
                Framework.log.logMessage(Framework.network, Log.LEVEL_ERROR, "Unable to save new root certificate information.");
                throw e;
            }
        }
    }

    private void createNewConfiguration(){
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Creating new configuration.");
        // Create a new configuration with a new name, principal, and pass
        String peerName = javax.swing.JOptionPane.showInputDialog("Enter your name:", "CI.KMR - <unknown peer>");
        if(peerName == null || peerName.trim().length() == 0) peerName = "CI.KMR - <unknown peer>";

        networkConfigurator.setName(peerName);
        networkConfigurator.setPrincipal("master");
        networkConfigurator.setPassword("thesis");
        //            networkConfigurator.setInfrastructureID("uuid-4E544E553A4E4F72B761B9202D204D6102");
        //            networkConfigurator.setInfrastructureName("CI.KMR");
        //            networkConfigurator.setInfrastructureDescription("NTNU:Norway - Master thesis 2007 - Christian Indahl & Kjell Martin Rud");
        try {
            networkConfigurator.save();
        } catch (IOException io) {
            Framework.log.logMessage(Framework.network, Log.LEVEL_ERROR, "Unable to save configuration.");
        }
    }

    private void connectToNetPeerGroup() throws PeerGroupException{
        if(connectionStatus.isConnectedToNetPeerGroup()) return;
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Connecting to net peer group.");
        connectionStatus.setStatus(ConnectionStatus.STATUS_CONNECTING_TO_NET_PEER_GROUP);
        connectionStatus.fireConnectionStatusUpdated();

        NetPeerGroupFactory netPeerGroupFactory;
        try{
            netPeerGroupFactory = new NetPeerGroupFactory(networkConfigurator.getPlatformConfig(), networkConfigurator.getStoreHome());
        }catch(PeerGroupException pgEx){
            Framework.log.logMessage(Framework.network, Log.LEVEL_ERROR, "Unable to become member of default net peer group.");
            throw pgEx;
        }
        netPeerGroup = netPeerGroupFactory.getInterface();
    }

    private void connectToPeerGroup() throws PeerGroupException{
        Framework.log.logMessage(Framework.network, Log.LEVEL_DEBUG, "Connecting to peer group.");
        connectionStatus.setStatus(ConnectionStatus.STATUS_CONNECTING_TO_PEER_GROUP);
        connectionStatus.fireConnectionStatusUpdated();


        createPeerGroup();

        peerGroup.init(netPeerGroup, peerGroup.getPeerGroupID(), peerGroup.getPeerAdvertisement());
        //peerGroup.startApp(null);
        connectionStatus.setPeerName(peerGroup.getPeerName());

        DiscoveryService discoveryService = peerGroup.getDiscoveryService();

/**
        try{
            discoveryService.publish(peerGroup.getPeerAdvertisement());
            discoveryService.remotePublish(peerGroup.getPeerAdvertisement());
        } catch(Exception ex){
            ex.printStackTrace();
        }
**/

        try{
            discoveryService.flushAdvertisements(null, DiscoveryService.GROUP);
        } catch(IOException ioEx){
            ioEx.printStackTrace();
        }
        try{
            discoveryService.flushAdvertisements(null, DiscoveryService.PEER);
        } catch(IOException ioEx){
            ioEx.printStackTrace();
        }
        try{
            discoveryService.flushAdvertisements(null, DiscoveryService.ADV);
        } catch(IOException ioEx){
            ioEx.printStackTrace();
        }

    }

    private void createPeerGroup() throws PeerGroupException{
        if(peerGroup != null) return;

        PeerGroupID peerGroupID = net.jxta.id.IDFactory.newPeerGroupID(new String("NTNU:Norway - Master thesis 2007 - Christian Indahl & Kjell Martin Rud").getBytes());

        //            ID("uuid-4E544E553A4E4F72B761B9202D204D6102");
        //            Name("CI.KMR");
        //            Description("NTNU:Norway - Master thesis 2007 - Christian Indahl & Kjell Martin Rud");
        Advertisement peerGroupAdvertisement = null;

        try{
            Enumeration e = netPeerGroup.getDiscoveryService().getLocalAdvertisements(DiscoveryService.GROUP, null, null);
            while(e.hasMoreElements()){
                PeerGroupAdvertisement adv = (PeerGroupAdvertisement)e.nextElement();
                if(adv.getPeerGroupID().equals(peerGroupID)){
                    peerGroupAdvertisement = adv;
                    break;
                }
            }
        } catch(IOException ex){}

        if(peerGroupAdvertisement == null){
            Framework.log.logMessage(Framework.network, Log.LEVEL_INFORMATIVE, "Creating peer group advertisement.");
            try{
                peerGroupAdvertisement = netPeerGroup.getAllPurposePeerGroupImplAdvertisement();
            } catch(Exception ex){
                Framework.log.logMessage(Framework.network, Log.LEVEL_SEVERE, "Unable to create peer group.");
                throw new PeerGroupException("Unable to create peer group advertisement.");
            }
            peerGroup = netPeerGroup.newGroup(peerGroupID, peerGroupAdvertisement, "CI.KMR", "NTNU:Norway - Master thesis 2007 - Christian Indahl & Kjell Martin Rud");
        }else{
            Framework.log.logMessage(Framework.network, Log.LEVEL_INFORMATIVE, "Found peer group advertisement locally.");
            peerGroup = netPeerGroup.newGroup(peerGroupAdvertisement);
        }
    }

    protected ConnectionStatus getConnectionStatus(){
        return connectionStatus;
    }

//    protected PeerGroup getNetPeerGroup(){
//        return netPeerGroup;
//    }

//    protected NetworkConfigurator getNetworkConfigurator(){
//        return networkConfigurator;
//    }

    public PeerGroup getPeerGroup(){
        return peerGroup;
    }

}
