package edu.ntnu.cikmr.ontologies;

import com.hp.hpl.jena.util.iterator.ExtendedIterator;
import com.hp.hpl.jena.ontology.Individual;
import com.hp.hpl.jena.ontology.OntClass;
import edu.ntnu.cikmr.Framework;

public class OntologyUtils {

    public static int getDownwardDistance(OntClass concept, String individual){
        return getDownwardDistance(concept, individual, 0);
    }

    private static int getDownwardDistance(OntClass root, String individualLocalName, int distance){
        int NOT_FOUND = -1;

        ExtendedIterator classes = root.listSubClasses(true);
        while(classes.hasNext()){
            OntClass child = (OntClass)classes.next();
            int d = getDownwardDistance(child, individualLocalName, distance + 1);
            if(d > NOT_FOUND) return d;
        }
        ExtendedIterator individuals = root.listInstances();
        while(individuals.hasNext()){
            Individual individual = (Individual)individuals.next();
            String localName = individual.getLocalName();
            if(localName.equals(individualLocalName)) return distance;
        }
        return NOT_FOUND;
    }

    public static int getUpwardDistance(OntClass concept, String individual){
        return getUpwardDistance(concept, individual, 0);
    }

    public static int getUpwardDistance(OntClass root, String individualLocalName, int distance){
        int NOT_FOUND = -1;

        ExtendedIterator classes = root.listSuperClasses(true);
        while(classes.hasNext()){
            OntClass parent = (OntClass)classes.next();
            int d = getDownwardDistance(parent, individualLocalName, distance + 1);
            if(d > NOT_FOUND) return d;
        }
        ExtendedIterator individuals = root.listInstances();
        while(individuals.hasNext()){
            Individual individual = (Individual)individuals.next();
            String localName = individual.getLocalName();
            if(localName.equals(individualLocalName)) return distance;
        }
        return NOT_FOUND;
    }

    public static OntClass getConcept(String localName){
        return Framework.ontologies.getConceptByLocalName(localName);
    }

    public static Individual getIndividual(String localName){
        return Framework.ontologies.getInstanceByLocalName(localName);
    }

    public static OntClass findOwner(String concept, String individual){
        return findOwner(getConcept(concept), individual);
    }

    private static OntClass findOwner(OntClass ontClass, String individualLocalName){
        ExtendedIterator classes = ontClass.listSubClasses(true);
        while(classes.hasNext()){
            OntClass child = (OntClass)classes.next();
            OntClass parent = findOwner(child, individualLocalName);
            if(parent != null) return parent;
        }
        ExtendedIterator individuals = ontClass.listInstances();
        while(individuals.hasNext()){
            Individual individual = (Individual)individuals.next();
            String localName = individual.getLocalName();
            if(localName.equals(individualLocalName)) return ontClass;
        }
        return null;
    }

}
