package edu.ntnu.cikmr.services;

import java.util.*;

import edu.ntnu.cikmr.*;

import net.jxta.peergroup.*;
import edu.ntnu.cikmr.log.Log;
import java.io.IOException;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class ServiceManager extends AbstractService{
    private ArrayList services;
    private boolean started;

    public ServiceManager(){
        services = new ArrayList();
        started = false;
    }

    public Service get(Class service){
        for(int i = 0; i < services.size(); i++){
            try {
                Service s = (Service) services.get(i);
                if(service.isAssignableFrom(s.getClass())){
                    Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Found instance of " + service.getSimpleName() + ".");
                    return s;
                }
            } catch (Exception ex) {}
        }
        Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Unable to find instance of " + service.getSimpleName() + ".");
        return null;
    }

    public void add(Service s){
        services.add(s);
        s.init(this);
        Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Added " + s.getClass().getSimpleName() + ".");
        if(started) start(s, Framework.network.getPeerGroup());
    }

    public void remove(Service s){
        if(services.remove(s)){
            if(started) stop(s);
            s.init(null);
            Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Removed " + s.getClass().getSimpleName() + ".");
        }
    }

    public synchronized void startServices(){
        if(started) return;
        PeerGroup peerGroup = Framework.network.getPeerGroup();
        synchronized(services){
            for(int i = 0; i < services.size(); i++){
                start((Service)services.get(i), peerGroup);
            }
        }
        started = true;
    }

    private void start(Service service, PeerGroup peerGroup){
        try {
            Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Starting " + service.getClass().getSimpleName() + ".");
            service.start(peerGroup);
            fireServiceStarted(service);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public synchronized void stopServices(){
        if(!started) return;
        synchronized(services){
            for(int i = services.size() - 1; i >= 0; i--){
                stop((Service)services.get(i));
            }
        }
        started = false;
    }

    private void stop(Service service){
        try {
            Framework.log.logMessage(this, Log.LEVEL_DEBUG, "Stopping " + service.getClass().getSimpleName() + ".");
            service.stop();
            fireServiceStopped(service);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void init(ServiceManager manager) {}

    public void start(PeerGroup peerGroup) throws IOException {}

    public void stop() throws IOException {}

    public boolean isStarted() {
        return true;
    }

}
