package edu.ntnu.cikmr.services.peers;

import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;

import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.log.Log;
import edu.ntnu.cikmr.services.AbstractService;
import edu.ntnu.cikmr.services.ServiceManager;
import net.jxta.discovery.*;
import net.jxta.peergroup.PeerGroup;
import net.jxta.protocol.PeerAdvertisement;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class PeersService extends AbstractService{
    private ServiceManager serviceManager;
    private PeerGroup peerGroup;
    private DiscoveryService discoveryService;
    private String myPeerID;
    private Hashtable peers;
    private boolean isMonitoring;
    private int timeToLive;
    private int discoveryInterval;
    private PeerTimeoutRunner peerTimeoutRunner;
    private PeerDiscoveryRunner peerDiscoveryRunner;
    private Peer me;


    public PeersService(){
        peers = new Hashtable();
    }

    private synchronized void discoverPeer(PeerAdvertisement peer){
        String peerID = peer.getPeerID().getUniqueValue().toString();
        if(!peerID.equals(myPeerID)){
            if(peers.containsKey(peerID)){
                PeerWrapper peerWrapper = (PeerWrapper)peers.get(peerID);
                PeerAdvertisement peerAdv = (PeerAdvertisement)peer.clone();
                peerWrapper.peer.setAdvertisement(peerAdv);
                peerWrapper.peer.setName(peerAdv.getName());
                peerWrapper.peer.setDescription(peerAdv.getDescription());
                peerWrapper.updated = true;
            }else{
                PeerWrapper peerWrapper = new PeerWrapper();
                peerWrapper.peer = new Peer();
                PeerAdvertisement peerAdv = (PeerAdvertisement)peer.clone();
                peerWrapper.peer.setAdvertisement(peerAdv);
                peerWrapper.peer.setID(peerID);
                peerWrapper.peer.setName(peerAdv.getName());
                peerWrapper.peer.setDescription(peerAdv.getDescription());
                peerWrapper.updated = true;
                peers.put(peerID, peerWrapper);
                Framework.log.logMessage(this, Log.LEVEL_INFORMATIVE, "Peer found (" + peerWrapper.peer.getName() + ").");
                firePeerAdded(peerWrapper.peer);
            }
            try{
                discoveryService.flushAdvertisement(peer);
            } catch(IOException ex){}
        }
    }

    public Peer[] getPeers(){
        synchronized(peers){
            Peer[] retPeers = new Peer[peers.size()];
            int index = 0;
            Enumeration e = peers.elements();
            while(e.hasMoreElements()){
                retPeers[index++] = ((PeerWrapper)e.nextElement()).peer;
            }
            return retPeers;
        }
    }

    public Peer getMe(){
        return isStarted() ? me : null;
    }

    public void init(ServiceManager manager){
        serviceManager = manager;
        isMonitoring = false;
        discoveryInterval = 10000;
        timeToLive = discoveryInterval * 2;
    }

    public boolean isStarted(){
        return (isMonitoring && peerTimeoutRunner != null && peerDiscoveryRunner != null);
    }

    public synchronized void start(PeerGroup peerGroup) throws IOException, IllegalStateException, IllegalArgumentException{
        if(peerGroup == null) throw new IllegalArgumentException("Peer group can not be null.");
        if(isMonitoring == true || peerTimeoutRunner != null || peerDiscoveryRunner != null) throw new IllegalStateException("Service already running.");
        this.peerGroup = peerGroup;
        try{
            myPeerID = peerGroup.getPeerID().getUniqueValue().toString();
            discoveryService = peerGroup.getDiscoveryService();
            isMonitoring = true;

            peerTimeoutRunner = new PeerTimeoutRunner();
            peerTimeoutRunner.start();

            peerDiscoveryRunner = new PeerDiscoveryRunner();
            peerDiscoveryRunner.start();

            me = new Peer(peerGroup.getPeerAdvertisement());

            Framework.log.logMessage(this, Log.LEVEL_DEBUG, "Started.");
            fireServiceStarted(this);
        } catch(Exception ex){
            throw new IOException("Unable to start peers service.");
        }
    }

    public synchronized void stop() throws IOException, IllegalStateException{
        if(isMonitoring == false && peerTimeoutRunner == null) throw new IllegalStateException("Service already stopped.");
        isMonitoring = false;
        peers = new Hashtable();
        try{
            peerDiscoveryRunner.interrupt();
            peerDiscoveryRunner.join(5000);
        }catch(Exception ex){}
        try{
            peerTimeoutRunner.interrupt();
            peerTimeoutRunner.join(5000);
        }catch(Exception ex){}
        peerDiscoveryRunner = null;
        peerTimeoutRunner = null;
        me = null;
        Framework.log.logMessage(this, Log.LEVEL_DEBUG, "Stopped.");
        fireServiceStopped(this);
    }

//------------------------------------------------------------------------------

    private class PeerWrapper{
        Peer peer;
        boolean updated = false;
    }

    private class PeerDiscoveryRunner extends Thread implements DiscoveryListener{
        public void run() {
            Framework.log.logMessage(PeersService.this, Log.LEVEL_DEBUG, "Discovery runner started.");
            while(isMonitoring){
                Framework.log.logMessage(PeersService.this, Log.LEVEL_INFORMATIVE, "Sending discovery request.");
                discoveryService.getRemoteAdvertisements(null, DiscoveryService.PEER, null, null, 1000, this);
                try {
                    Thread.sleep(discoveryInterval);
                } catch (Exception ex) {}
            }
            Framework.log.logMessage(PeersService.this, Log.LEVEL_DEBUG, "Discovery runner stopped.");
        }

        public void discoveryEvent(DiscoveryEvent discoveryEvent){
            if(!isStarted()) return;
            Enumeration peers = discoveryEvent.getSearchResults();
            while(peers.hasMoreElements()){
                try{
                    discoverPeer((PeerAdvertisement)peers.nextElement());
                } catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        }

    }

    private class PeerTimeoutRunner extends Thread{
        public void run(){
            Framework.log.logMessage(PeersService.this, Log.LEVEL_DEBUG, "Timeout runner started.");
            while(isMonitoring){
                try{
                    Thread.sleep(timeToLive);
                }catch(Exception ex){}
                try{
                    Enumeration p = peers.elements();
                    while(p.hasMoreElements()){
                        PeerWrapper peerWrapper = (PeerWrapper)p.nextElement();
                        if(peerWrapper.updated == true){
                            peerWrapper.updated = false;
                        } else{
                            peers.remove(peerWrapper.peer.getID());
                            Framework.log.logMessage(PeersService.this, Log.LEVEL_INFORMATIVE, "Peer removed (" + peerWrapper.peer.getName() + ").");
                            firePeerRemoved(peerWrapper.peer);
                        }
                    }
                }catch(Exception ex){}
            }
            Framework.log.logMessage(PeersService.this, Log.LEVEL_DEBUG, "Timeout runner stopped.");
        }
    }

//------------------------------------------------------------------------------

    public void addPeerListener(PeerListener l){
        listenerList.add(PeerListener.class, l);
    }

    public void removePeerListener(PeerListener l){
        listenerList.remove(PeerListener.class, l);
    }

    private void firePeerAdded(Peer peer){
        if(!isMonitoring) return;
        PeerListener[] listeners = (PeerListener[])listenerList.getListeners(PeerListener.class);
        PeerEvent event = null;
        for(int i = listeners.length - 1; i >= 0; i--){
            if(event == null) event = new PeerEvent(this, peer);
            listeners[i].peerAdded(event);
        }
    }

    private void firePeerRemoved(Peer peer){
        if(!isMonitoring) return;
        PeerListener[] listeners = (PeerListener[])listenerList.getListeners(PeerListener.class);
        PeerEvent event = null;
        for(int i = listeners.length - 1; i >= 0; i--){
            if(event == null) event = new PeerEvent(this, peer);
            listeners[i].peerRemoved(event);
        }
    }

    //------------------------------------------------------------------------------

}
