package edu.ntnu.cikmr.services.resources;

import java.text.*;
import java.util.*;

import com.hp.hpl.jena.ontology.*;
import edu.ntnu.cikmr.*;
import org.w3c.dom.*;

public class ResourceDescription {
    private Hashtable<String,ResourceMethod> methods;
    private String description;
    private Individual type;
    private Individual location;

    public ResourceDescription(){
        methods = new Hashtable<String,ResourceMethod>(1);
    }

    public void setDescription(String description){
        this.description = description;
    }

    public void setLocation(Individual location) {
        this.location = location;
    }

    public void setType(Individual type) {
        this.type = type;
    }

    public String getDescription(){
        return description;
    }

    public Individual getLocation() {
        return location;
    }

    public Individual getType() {
        return type;
    }

    public ResourceMethod[] getMethods(){
        return methods.values().toArray(new ResourceMethod[methods.size()]);
    }

    public ResourceMethod getMethod(String name){
        return methods.get(name);
    }

    public void addMethod(ResourceMethod method){
        methods.put(method.getMethodName(), method);
    }

    public void removeMethod(String name){
        methods.remove(name);
    }

    public static ResourceDescription fromXML(Node resourceNode) throws ParseException{
        ResourceDescription resourceDescription = new ResourceDescription();

        try {
            String name = resourceNode.getNodeName();
            if(!"resource".equalsIgnoreCase(name)){
                NodeList nodeList = resourceNode.getChildNodes();
                for(int i = 0; i < nodeList.getLength(); i++){
                    Node node = nodeList.item(i);
                    name = node.getNodeName();
                    if("resource".equalsIgnoreCase(name)){
                        resourceNode = node;
                        break;
                    }
                }
                if(!"resource".equalsIgnoreCase(name)) return null;
            }

            String type = null;
            String location = null;

            NodeList nodeList = resourceNode.getChildNodes();
            for (int i = 0; i < nodeList.getLength(); i++) {
                Node node = nodeList.item(i);
                String nodeName = node.getNodeName();
                if (nodeName.equalsIgnoreCase("type")) {
                    type = getTextFromNode(node);
                } else if (nodeName.equalsIgnoreCase("description")) {
                    resourceDescription.setDescription(getTextFromNode(node));
                } else if (nodeName.equalsIgnoreCase("location")) {
                    location = getTextFromNode(node);
                } else if (nodeName.equalsIgnoreCase("method")) {
                    ResourceMethod method = ResourceMethod.fromXML(node);
                    if (method != null) resourceDescription.addMethod(method);
                }
            }

            Individual typeIndividual = Framework.ontologies.getInstanceByLocalName(type);
            Individual locationIndividual = Framework.ontologies.getInstanceByLocalName(location);

            if(typeIndividual == null) throw new Exception("Unable to find ontology individual(s).");

            resourceDescription.setType(typeIndividual);
            resourceDescription.setLocation(locationIndividual);

        } catch (ParseException ex){
            throw ex;
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new ParseException("Unable to parse resource description.", 0);
        }
        return resourceDescription;
    }

    public static String toXML(ResourceDescription resourceDescription){
        ResourceMethod[] methods = resourceDescription.getMethods();

        String xml = "<resource>\n";
        xml += "<type>" + resourceDescription.getType().getLocalName() + "</type>\n";
        xml += "<description>" + resourceDescription.getDescription() + "</description>\n";
        xml += "<location>" + (resourceDescription.getLocation() == null ? "" : resourceDescription.getLocation().getLocalName()) + "</location>\n";
        if(methods != null && methods.length > 0) for (int i = 0; i < methods.length; i++) xml += ResourceMethod.toXML(methods[i]) + "\n";
        xml += "</resource>";
        return xml;
    }

    private static String getTextFromNode(Node node){
        String text = "";
        NodeList nodeList = node.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node childNode = nodeList.item(i);
            if(childNode.getNodeType() == Node.TEXT_NODE) text += childNode.getNodeValue();
        }
        return text;
    }

}
