
package edu.ntnu.cikmr.services.resources;

import java.text.*;
import java.util.*;

import org.w3c.dom.*;
import java.lang.reflect.*;

public class ResourceMethod {
//------------------------------------------------------------------------------
    protected Resource resource;
//------------------------------------------------------------------------------
    private ArrayList<String> parameters;
    private String returnType;
    private String methodName;

    public ResourceMethod() {
        parameters = new ArrayList<String>();
    }

    public void setMethodName(String name){
        methodName = name;
    }

    public String getMethodName(){
        return methodName;
    }

    public void addParameter(String type){
        parameters.add(type);
    }

    public void addParameter(String type, int index){
        parameters.add(index, type);
    }

    public void removeParameter(int index){
        parameters.remove(index);
    }

    public String[] getParameters(){
        return parameters.toArray(new String[parameters.size()]);
    }

    public void setReturnType(String returnType){
        this.returnType = returnType;
    }

    public String getReturnType(){
        return returnType;
    }

    public static ResourceMethod fromXML(Node methodNode) throws ParseException{
        ResourceMethod resourceMethod = new ResourceMethod();

        try{
            NamedNodeMap attributeMap = methodNode.getAttributes();
            Node nameNode = attributeMap.getNamedItem("name");
            resourceMethod.setMethodName(nameNode.getNodeValue());

            Node returnTypeNode = attributeMap.getNamedItem("returnType");
            resourceMethod.setReturnType(returnTypeNode.getNodeValue());

            if(methodNode.hasChildNodes()){
                NodeList nodeList = methodNode.getChildNodes();
                for (int i = 0; i < nodeList.getLength(); i++) {
                    Node parameter = nodeList.item(i);
                    if("parameter".equalsIgnoreCase(parameter.getNodeName())){
                        Node parameterValueNode = parameter.getAttributes().getNamedItem("type");
                        if(parameterValueNode != null) resourceMethod.addParameter(parameterValueNode.getNodeValue());
                    }
                }
            }
        }catch(Exception ex){
            ex.printStackTrace();
            throw new ParseException("Could not create a ResourceMethod from XML node.", 0);
        }
        return resourceMethod;
    }

    public static String toXML(ResourceMethod method){
        String[] params = method.getParameters();

        String xml = "<method name=\"" + method.getMethodName() + "\" returnType=\"" + method.getReturnType() + "\"";
        if(params != null && params.length > 0){
            xml += ">\n";
            for (int i = 0; i < params.length; i++) xml += "<parameter type=\"" + params[i] + "\"/>\n";
            xml += "</method>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public InvocationResult invokeMethod(Object[] parameters) throws InvocationTargetException{
        if(resource == null) throw new InvocationTargetException(new NullPointerException("No resource information object."));
        return resource.invokeMethod(this, parameters);
    }

}
