package edu.ntnu.cikmr.services.temperature;

import java.io.*;

import edu.ntnu.cikmr.services.*;
import edu.ntnu.cikmr.services.resources.*;
import net.jxta.peergroup.*;
import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.services.resources.actuators.*;

public class TemperatureActuatorGreenRoomResource extends AbstractResourceService implements SimpleNumericActuator{
    private boolean isStarted;
    private ResourceDescription resourceDescription;
    private int level = 0;
    private static final int MAX_LEVEL = 10;
    private static final int MIN_LEVEL = 0;
    private TemperatureAffectuator temperatureAffectuator;
    /**
     * init
     *
     * @param manager ServiceManager
     * @todo Implement this edu.ntnu.cikmr.services.Service method
     */
    public void init(ServiceManager manager) {
        resourceDescription = new ResourceDescription();
        resourceDescription.setType(Framework.ontologies.getInstanceByLocalName("Temperature_actuator"));
        resourceDescription.setDescription("Heating oven in the green room.");
        resourceDescription.setLocation(Framework.ontologies.getInstanceByLocalName("Green_room"));

        ResourceMethod setLevelMethod = new ResourceMethod();
        setLevelMethod.setMethodName("setLevel");
        setLevelMethod.setReturnType("void");
        setLevelMethod.addParameter("int");
        resourceDescription.addMethod(setLevelMethod);

        ResourceMethod getLevelMethod = new ResourceMethod();
        getLevelMethod.setMethodName("getLevel");
        getLevelMethod.setReturnType("int");
        resourceDescription.addMethod(getLevelMethod);

        setLevelMethod = new ResourceMethod();
        setLevelMethod.setMethodName("setValue");
        setLevelMethod.setReturnType("void");
        setLevelMethod.addParameter("double");
        resourceDescription.addMethod(setLevelMethod);
    }

    public int getLevel(){
        return level;
    }

    public void setLevel(int level){
        this.level = level;
        if(this.level > MAX_LEVEL) this.level = MAX_LEVEL;
        if(this.level < MIN_LEVEL) this.level = MIN_LEVEL;
    }

    /**
     * isStarted
     *
     * @return boolean
     * @todo Implement this edu.ntnu.cikmr.services.Service method
     */
    public boolean isStarted() {
        return isStarted;
    }

    /**
     * start
     *
     * @param peerGroup PeerGroup
     * @throws IOException
     * @todo Implement this edu.ntnu.cikmr.services.Service method
     */
    public void start(PeerGroup peerGroup) throws IOException {
        isStarted = true;
        try {
            startResource(peerGroup);
        } catch (InstantiationException ex) {
            isStarted = false;
            throw new IOException("Unable to start resource.");
        }
        temperatureAffectuator = new TemperatureAffectuator();
        fireServiceStarted(this);
    }

    /**
     * stop
     *
     * @throws IOException
     * @todo Implement this edu.ntnu.cikmr.services.Service method
     */
    public void stop() throws IOException {
        isStarted = false;
        stopResource();
        try {
            temperatureAffectuator.run = false;
            temperatureAffectuator.interrupt();
            temperatureAffectuator.join(5000);
        } catch (Exception ex) {}
        temperatureAffectuator = null;
        fireServiceStopped(this);
    }

    public ResourceDescription getResourceDescription() {
        return resourceDescription;
    }

    public void setValue(double d) {
        setLevel((int)d);
    }

    private class TemperatureAffectuator extends Thread{
        private boolean run;
        private int iterationDelay = 1000;
        private double decreaseLevel = -2.0;

        private TemperatureAffectuator(){
            run = true;
            start();
        }

        public void run(){
            while(run){
                double changeLevel = level > 0 ? (double)level : decreaseLevel;
                double change = changeLevel / (((double)iterationDelay) / 1000.0 * 60.0);
                if(change < 0 && Temperature.temperatureGreenRoom > Temperature.outsideTemperature) Temperature.temperatureGreenRoom += change;
                if(Temperature.temperatureGreenRoom < Temperature.outsideTemperature) Temperature.temperatureGreenRoom = Temperature.outsideTemperature;
                if(change > 0 && Temperature.temperatureGreenRoom < Temperature.ovenMaxTemperature) Temperature.temperatureGreenRoom += change;
                if(Temperature.temperatureGreenRoom > Temperature.ovenMaxTemperature) Temperature.temperatureGreenRoom = Temperature.ovenMaxTemperature;
                try{
                    Thread.sleep(iterationDelay);
                }catch(Exception ex){}
            }
        }

    }

}
