package edu.ntnu.cikmr.workflow;

import java.util.*;

import com.hp.hpl.jena.ontology.*;
import edu.ntnu.cikmr.*;
import org.w3c.dom.*;

public class Activity extends ExecutableActivity{
    private boolean result;

    private ArrayList<Statement> statements;

    public Activity(){
        statements = new ArrayList<Statement>();
    }

    public void fromXML(Node n) {
        NamedNodeMap nodeMap = n.getAttributes();
        Node id = nodeMap.getNamedItem("id");
        setId((id == null) ? getParent().createUniqueAcitivyID() : id.getNodeValue());
        if(getId().trim().length() == 0) setId(getParent().createUniqueAcitivyID());

        Node type = nodeMap.getNamedItem("type");
        String typeString = type == null ? null : type.getNodeValue();
        setType(typeString == null ? null : Framework.ontologies.getInstanceByLocalName(typeString));

        Statement[] foundStatements = Activity.getStatements(n, this);
        if(foundStatements == null) return;
        for(int i = 0; i < foundStatements.length; i++) statements.add(foundStatements[i]);
    }

    public String toXML() {
        String xml = "<activity" + (getId() == null ? "" : " id=\"" + getId() + "\"") + (getType() == null ? "" : " type=\"" + getType().getLocalName() + "\"");
        if(statements.size() > 0){
            xml += ">\n";
            Iterator<Statement> iterator = statements.iterator();
            while(iterator.hasNext()){
                xml += iterator.next().toXML() + "\n";
            }
            xml += "</activity>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public String toTypeXML(String location, String pipeID, String workflowID){
        return getType() == null ? null : "<activity id=\"" + getId() + "\" type=\"" + getType().getLocalName() + "\" location=\"" + location + "\" pipeID=\"" + pipeID + "\" workflowID=\"" + workflowID + "\"/>";
    }

    public boolean execute() {
        getLog().logMessage(this, getLog().LEVEL_INFORMATIVE, "Executing.");
        Iterator<Statement> iterator = statements.iterator();
        while(iterator.hasNext()){
            Statement statement = iterator.next();
            if(!statement.execute()){
                return result = false;
            }
            if(isHalted()){
                return result = false;
            }
        }
        return result = true;
    }

    public boolean getResult() {
        return result;
    }

    public int getStatementCount(){
        return statements.size();
    }

    public Statement getStatement(int index){
        return statements.get(index);
    }

    public void removeStatement(int index){
        statements.remove(index);
    }

    public Iterator<Statement> getStatementIterator(){
        return statements.iterator();
    }

    public static Statement[] getStatements(Node n, Activity parent){
        ArrayList<Statement> statements = new ArrayList<Statement>();

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            String name = childNode.getNodeName();
            Statement statement = null;
            if(name.equalsIgnoreCase("execute")){
                statement = new Execute();
            }else if(name.equalsIgnoreCase("wait")){
                statement = new Wait();
            }else if(name.equalsIgnoreCase("compare")){
                statement = new Compare();
            }else if(name.equalsIgnoreCase("condition")){
                statement = new Condition();
            }else if(name.equalsIgnoreCase("findResource")){
                statement = new FindResource();
            }else if(name.equalsIgnoreCase("isDefined")){
                statement = new IsDefined();
            }else if(name.equalsIgnoreCase("setDefined")){
                statement = new SetDefined();
            }else if(name.equalsIgnoreCase("useResource")){
                statement = new UseResource();
            }
            if(statement != null){
                statement.setParent(parent);
                statement.fromXML(childNode);
                statements.add(statement);
            }
        }
        return statements.toArray(new Statement[statements.size()]);
    }

}
