package edu.ntnu.cikmr.workflow;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

/**
 *
 * <p>Title: Compare statement</p>
 *
 * <p>Description: Compares two values.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class Compare extends Statement{
    private String ls;
    private String rs;
    private String type;
    private boolean negate;
    private boolean result;

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node ls = nodeMap.getNamedItem("ls");
        this.ls = ls == null ? null : ls.getNodeValue();
        Node rs = nodeMap.getNamedItem("rs");
        this.rs = rs == null ? null : rs.getNodeValue();
        Node type = nodeMap.getNamedItem("type");
        this.type = type == null ? null : type.getNodeValue();
        Node negate = nodeMap.getNamedItem("negate");
        this.negate = negate == null ? false : new Boolean(type.getNodeValue()).booleanValue();
    }

    public String toXML(){
        return "<compare ls=\"" + ls + "\" rs=\"" + rs + "\" type=\"" + type + "\"/>";
    }

    public boolean execute(){
        if(type.equalsIgnoreCase("StringEqualsIgnoreCase")){
            return compareStringEqualsIgnoreCase();
        }else if(type.equalsIgnoreCase("StringEquals")){
            return compareStringEquals();
        }else if(type.equalsIgnoreCase("NumericEquals")){
            return compareNumericEquals();
        }else if(type.equalsIgnoreCase("NumericGreater")){
            return compareNumericGreater();
        }else if(type.equalsIgnoreCase("NumericGreaterEquals")){
            return compareNumericGreaterEquals();
        }else if(type.equalsIgnoreCase("NumericLess")){
            return compareNumericLess();
        }else if(type.equalsIgnoreCase("NumericLessEquals")){
            return compareNumericLessEquals();
        }else if(type.equalsIgnoreCase("Boolean")){
            return compareBoolean();
        }
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Invalid type.");
        return result = false;
    }

    private boolean compareStringEqualsIgnoreCase(){
        try{
            result = resolve(ls).equalsIgnoreCase(resolve(rs));
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareStringEquals(){
        try{
            result = resolve(ls).equals(resolve(rs));
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareNumericEquals(){
        try{
            double lsd = Double.parseDouble(resolve(ls));
            double rsd = Double.parseDouble(resolve(rs));
            result = lsd == rsd;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareNumericGreater(){
        try{
            double lsd = Double.parseDouble(resolve(ls));
            double rsd = Double.parseDouble(resolve(rs));
            result = lsd > rsd;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareNumericGreaterEquals(){
        try{
            double lsd = Double.parseDouble(resolve(ls));
            double rsd = Double.parseDouble(resolve(rs));
            result = lsd >= rsd;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareNumericLess(){
        try{
            double lsd = Double.parseDouble(resolve(ls));
            double rsd = Double.parseDouble(resolve(rs));
            result = lsd < rsd;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareNumericLessEquals(){
        try{
            double lsd = Double.parseDouble(resolve(ls));
            double rsd = Double.parseDouble(resolve(rs));
            result = lsd <= rsd;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    private boolean compareBoolean(){
        try{
            result = new Boolean(resolve(ls)).booleanValue() == new Boolean(resolve(rs)).booleanValue();
            if(negate) result = !result;
        } catch(Exception ex){
            result = false;
        }
        if(negate) result = !result;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        return true;
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Returns the type of comparison to perform.
     *
     * @return type
     */
    public String getType(){
        return type;
    }

    /**
     * Returns the right side of the comparison
     *
     * @return value
     */
    public String getRs(){
        return rs;
    }

    /**
     * Return the left side of the comparison.
     *
     * @return value
     */
    public String getLs(){
        return ls;
    }

    /**
     * Sets the left side value or variable to compare.
     *
     * @param ls
     */
    public void setLs(String ls){
        this.ls = ls;
    }

    /**
     * Sets the right side value or variable to compare.
     *
     * @param rs
     */
    public void setRs(String rs){
        this.rs = rs;
    }

    /**
     * Sets the type of comparison to perform.
     *
     * @param type
     */
    public void setType(String type){
        this.type = type;
    }

}
