package edu.ntnu.cikmr.workflow;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import java.util.ArrayList;
import java.util.Iterator;

/**
 *
 * <p>Title: Condition statement</p>
 *
 * <p>Description: A conditional block</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class Condition extends Statement{
    private String iftrue;
    private String iffalse;
    private boolean andBinding;
    private boolean result;
    private ArrayList<Statement> statements;

    public Condition(){
        statements = new ArrayList<Statement>();
    }

    public void fromXML(Node n) {
        NamedNodeMap nodeMap = n.getAttributes();
        Node iftrue = nodeMap.getNamedItem("iftrue");
        this.iftrue = iftrue == null ? null : iftrue.getNodeValue();
        Node iffalse = nodeMap.getNamedItem("iffalse");
        this.iffalse = iffalse == null ? null : iffalse.getNodeValue();
        Node binding = nodeMap.getNamedItem("binding");
        String bindingString = binding == null ? null : binding.getNodeValue();
        andBinding = bindingString == null ? true : !bindingString.equalsIgnoreCase("or");

        Statement[] foundStatements = Activity.getStatements(n, getParent());
        if(foundStatements == null) return;
        for(int i = 0; i < foundStatements.length; i++) statements.add(foundStatements[i]);
    }

    public String toXML() {
        String xml = "<condition iftrue=\"" + iftrue + "\" iffalse=\"" + iffalse + "\" binding=\"" + (andBinding ? "and" : "or") + "\"";
        if(statements.size() > 0){
            xml += ">\n";
            Iterator<Statement> iterator = statements.iterator();
            while(iterator.hasNext()){
                xml += iterator.next().toXML() + "\n";
            }
            xml += "</condition>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public boolean execute(){
        if(andBinding){
            return executeAndBinding();
        }else{
            return executeOrBinding();
        }
    }

    private boolean executeAndBinding(){
        result = true;
        Iterator<Statement> iterator = statements.iterator();
        while(iterator.hasNext()){
            Statement statement = iterator.next();
            if(!statement.execute()){
                return result = false;
            }
            if(getParent().isHalted()){
                return result = false;
            }
            if(!statement.getResult()){
                result = false;
                break;
            }
            if(getParent().isHalted()) return result = false;
        }
        return executeIf();
    }

    private boolean executeOrBinding(){
        result = false;
        Iterator<Statement> iterator = statements.iterator();
        while(iterator.hasNext()){
            Statement statement = iterator.next();
            if(!statement.execute()){
                return result = false;
            }
            if(getParent().isHalted()){
                return result = false;
            }
            if(statement.getResult()){
                result = true;
                break;
            }
        }
        return executeIf();
    }

    private boolean executeIf(){
        String executeID = result ? iftrue : iffalse;

        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
        if(executeID != null && executeID.trim().length() > 0){
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing: " + executeID);
            Execute execute = new Execute();
            execute.setParent(getParent());
            execute.setExecuteID(executeID);
            if(execute.execute()){
                return true;
            }else{
                return false;
            }
        }else{
            return true;
        }
    }

    public boolean getResult() {
        return result;
    }

    /**
     * Return the id to execute if true.
     *
     * @return id
     */
    public String getIftrue(){
        return iftrue;
    }

    /**
     * Returns the id to execute if false
     *
     * @return id
     */
    public String getIffalse(){
        return iffalse;
    }

    /**
     * Return if the and or or binding is used.
     *
     * @return boolean
     */
    public boolean getAndBinding(){
        return andBinding;
    }

    /**
     * Sets if the binding should be and or or.
     *
     * @param andBinding boolean
     */
    public void setAndBinding(boolean andBinding){
        this.andBinding = andBinding;
    }

    /**
     * Sets the acitity to execute if false.
     *
     * @param iffalse id
     */
    public void setIffalse(String iffalse){
        this.iffalse = iffalse;
    }

    /**
     * Sets the activity to execute if true.,
     *
     * @param iftrue id
     */
    public void setIftrue(String iftrue){
        this.iftrue = iftrue;
    }

    public int getStatementCount(){
        return statements.size();
    }

    public Statement getStatement(int index){
        return statements.get(index);
    }

    public void removeStatement(int index){
        statements.remove(index);
    }

    public Iterator<Statement> getStatementIterator(){
        return statements.iterator();
    }

}
