package edu.ntnu.cikmr.workflow;

import com.hp.hpl.jena.ontology.Individual;
import edu.ntnu.cikmr.XMLRepresentation;
import edu.ntnu.cikmr.workflowenactment.WorkflowContext;
import edu.ntnu.cikmr.log.Log;

public abstract class ExecutableActivity implements Executable, XMLRepresentation{
    private Goal parent;
    private String id;
    private Individual type;
    private boolean halted;

    protected void setParent(Goal parent){
        this.parent = parent;
    }

    protected void setId(String id){
        this.id = id;
    }

    protected void setType(Individual type){
        this.type = type;
    }

    public Goal getParent(){
        return parent;
    }

    public Individual getType(){
        return type;
    }

    public String getId(){
        return id;
    }

    public boolean hasParent(){
        return parent != null;
    }

    public Log getLog(){
        Goal goal = this instanceof Goal ? (Goal)this : this.getParent();
        while(goal.getParent() != null) goal = goal.getParent();
        WorkflowContext context = goal.getWorkflowContext();
        if(context != null){
            return context.getLog();
        }else{
            return null;
        }
    }

    public boolean control(String command){
        Goal goal = this instanceof Goal ? (Goal)this : this.getParent();
        while(goal.getParent() != null) goal = goal.getParent();
        WorkflowContext context = goal.getWorkflowContext();
        if(context != null){
            return context.invokeControlCommand(command);
        }else{
            return false;
        }
    }

    public boolean isHalted(){
        if(halted) return true;
        if(parent != null) return parent.isHalted();
        if(!(this instanceof Goal)) return true;
        WorkflowContext context = ((Goal)this).getWorkflowContext();
        return context == null ? true : context.isHalted();
    }

    public void halt(){
        halted = true;
    }

    public void clearHalt(){
        halted = false;
    }

    public abstract String toTypeXML(String location, String pipeID, String workflowID);

}
