package edu.ntnu.cikmr.workflow;

import org.w3c.dom.*;

/**
 *
 * <p>Title: Execute statement</p>
 *
 * <p>Description: Executes an activity or control statement.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class Execute extends Statement{
    private String id;
    private String control;
    private boolean result;

    public boolean execute() {
        result = false;
        try{
            if(id != null && id.length() > 0) return executeActivity();
            if(control != null && control.length() > 0) return executeControl();
            return false;
        } catch(Exception ex){
            ex.printStackTrace();
            return false;
        }
    }

    private boolean executeActivity(){
        ExecutableActivity activity = this.getParent().getParent().getActivity(resolve(id));
        if(activity == null){
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Failed. No activity.");
            result = false;
            return false;
        }
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing: " + activity.getId());
        boolean b = activity.execute();
        if(b == false){
            result = false;
        }else{
            result = activity.getResult();
        }
        return b;
    }

    private boolean executeControl(){
        String command = resolve(control);
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing control:" + command);
        boolean res = result = this.getParent().control(command);
        return res;
    }

    public boolean getResult() {
        return result;
    }

    public void fromXML(Node n) {
        NamedNodeMap nodeMap = n.getAttributes();
        Node id = nodeMap.getNamedItem("id");
        this.id = id == null ? null : id.getNodeValue();
        Node control = nodeMap.getNamedItem("control");
        this.control = control == null ? null : control.getNodeValue();
    }

    /**
     * Returns the activity id to execute.
     *
     * @return id
     */
    public String getExecuteID(){
        return id;
    }

    /**
     * Return the workflow control statement to execute.
     *
     * @return control statement
     */
    private String getExecuteControl(){
        return control;
    }

    private boolean isExecuteControl(){
        return control != null;
    }

    /**
     * Sets the activity to execute.
     *
     * @param id
     */
    public void setExecuteID(String id) {
        this.id = id;
    }

    public boolean isExcuteID(){
        return id != null;
    }

    /**
     * Sets the workflow control statement to execute.
     *
     * @param control control statement
     */
    public void setExecuteControl(String control) {
        this.control = control;
    }

    public String toXML() {
        return "<execute" + (isExcuteID() ? " id=\"" + getExecuteID() + "\"" : "") + (isExecuteControl() ? " control=\"" + getExecuteControl() + "\"" : "") + "/>";
    }

}
