package edu.ntnu.cikmr.workflow;

import org.w3c.dom.Node;
import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.services.resources.Resource;
import org.w3c.dom.NamedNodeMap;

/**
 *
 * <p>Title: Find resource statement</p>
 *
 * <p>Description: Looks up a statement and stores it in a variable.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class FindResource extends Statement{
    private String type;
    private String location;
    private boolean local;
    private boolean result;
    private String name;

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node type = nodeMap.getNamedItem("type");
        this.type = type == null ? null : type.getNodeValue();
        Node location = nodeMap.getNamedItem("location");
        this.location = location == null ? null : location.getNodeValue();
        Node name = nodeMap.getNamedItem("name");
        this.name = name == null ? null : name.getNodeValue();
        Node local = nodeMap.getNamedItem("local");
        this.local = local == null ? false : new Boolean(local.getNodeValue()).booleanValue();
    }

    public String toXML(){
        return "<findResource type=\"" + type + "\"" + (hasLocation() ? " location=\"" + location + "\"" : "") + " name=\"" + name + "\" local=\"" + local + "\"/>";
    }

    public boolean execute(){
        try{
            result = false;
            Resource resource = null;
            resource = Framework.resources.findResource(resolve(type), hasLocation() ? resolve(location) : null, local);
            if(result = resource != null){
                getParent().getParent().setResource(name, resource);
            }
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + result);
            return true;
        } catch(Exception ex){
            ex.printStackTrace();
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Execution failed.");
            return false;
        }
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Return if the resource should be found locally
     *
     * @return boolean
     */
    public boolean isLocal(){
        return local;
    }

    /**
     * The location to look for the resource.
     *
     * @return location
     */
    public String getLocation(){
        return location;
    }

    public boolean hasLocation(){
        return (location != null && location.trim().length() > 0);
    }

    /**
     * Returns the type of resource to look for
     *
     * @return type
     */
    public String getType(){
        return type;
    }

    /**
     * Return the name of the variable to store the resource in.
     *
     * @return variable name
     */
    public String getName(){
        return name;
    }

    /**
     * Sets whether the resource should be searched for locally.
     *
     * @param local boolean
     */
    public void setLocal(boolean local){
        this.local = local;
    }

    /**
     * Sets the location to search for the resource.
     *
     * @param location
     */
    public void setLocation(String location){
        this.location = location;
    }

    /**
     * Sets the type of resource to search for.
     *
     * @param type
     */
    public void setType(String type){
        this.type = type;
    }

    /**
     * Sets the name of the variable to search for.
     *
     * @param name variable name
     */
    public void setName(String name){
        this.name = name;
    }


}
