package edu.ntnu.cikmr.workflow;

import java.util.*;

import edu.ntnu.cikmr.Framework;
import edu.ntnu.cikmr.services.resources.Resource;
import edu.ntnu.cikmr.workflowenactment.WorkflowContext;
import org.w3c.dom.*;
import edu.ntnu.cikmr.services.resources.ResourceUtils;
import java.util.Map.Entry;

public class Goal extends ExecutableActivity{
    private WorkflowContext workflowContext;
    private Incentive incentive;
    private Hashtable<String, ExecutableActivity> activities;
    private Hashtable<String, String> definitions;
    private Hashtable<String, ResourceMaintainer> resources;
    private boolean result;

    private IncentiveRunner incentiveRunner;

    public Goal(){
        activities = new Hashtable<String,ExecutableActivity>();
        definitions = new Hashtable<String,String>();
        resources = new Hashtable<String,ResourceMaintainer>();
    }

    public WorkflowContext getWorkflowContext(){
        return workflowContext;
    }

    public void setWorkflowContext(WorkflowContext workflowContext){
        this.workflowContext = workflowContext;
    }

    public ExecutableActivity getActivity(String name){
        ExecutableActivity activity = activities.get(name);
        if(activity == null && hasParent()) activity = getParent().getActivity(name);
        return activity;
    }

    public boolean hasActivity(String name){
        boolean hasActivity = activities.containsKey(name);
        if(!hasActivity && hasParent()) hasActivity = getParent().hasActivity(name);
        return hasActivity;
    }

    public ExecutableActivity addActivity(String name, ExecutableActivity activity){
        return activities.put(name, activity);
    }


    public String getDefinition(String name){
        String definition = definitions.get(name);
        if(definition == null && hasParent()) definition = getParent().getDefinition(name);
        return definition;
    }

    public boolean hasDefinition(String name){
        boolean hasDefinition = definitions.containsKey(name);
        if(!hasDefinition && hasParent()) hasDefinition = getParent().hasDefinition(name);
        return hasDefinition;
    }

    public String setDefinition(String name, String value){
        String oldValue = null;
        if(definitions.containsKey(name)){
            oldValue = definitions.put(name, value);
        }else if(hasDefinition(name)){
            getParent().setDefinition(name, value);
        }else{
            definitions.put(name, value);
        }
        return oldValue;
    }

    public Resource getResource(String name){
        ResourceMaintainer resourceMaintainer = resources.get(name);
        Resource resource = resourceMaintainer == null ? null : resourceMaintainer.getResource();
        if(resource == null && hasParent()) resource = getParent().getResource(name);
        return resource;
    }

    public boolean hasResource(String name){
        boolean hasResource = resources.containsKey(name);
        if(!hasResource && hasParent()) hasResource = getParent().hasResource(name);
        return hasResource;
    }

    public Resource setResource(String name, Resource value){
        ResourceMaintainer oldValue = null;
        if(resources.containsKey(name)){
            oldValue = resources.put(name, new ResourceMaintainer(name, value, resources));
        }else if(hasResource(name)){
            getParent().setResource(name, value);
        }else{
            resources.put(name, new ResourceMaintainer(name, value, resources));
        }
        return oldValue == null ? null : oldValue.getResource();
    }

    public void fromXML(Node n) {
        NamedNodeMap nodeMap = n.getAttributes();
        Node id = nodeMap.getNamedItem("id");
        setId(id == null ? createUniqueAcitivyID() : id.getNodeValue());
        if(getId().trim().length() == 0) setId(createUniqueAcitivyID());

        Node type = nodeMap.getNamedItem("type");
        String typeString = type == null ? null : type.getNodeValue();
        setType(typeString == null ? null : Framework.ontologies.getInstanceByLocalName(typeString));

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            ExecutableActivity activity = null;
            if(childNode.getNodeName().equalsIgnoreCase("goal")){
                activity = new Goal();
            }else if(childNode.getNodeName().equalsIgnoreCase("activity")){
                activity = new Activity();
            }else if(childNode.getNodeName().equalsIgnoreCase("incentive")){
                incentive = new Incentive();
                incentive.setParent(this);
                incentive.fromXML(childNode);
            }
            if(activity != null){
                activity.setParent(this);
                activity.fromXML(childNode);
                if(activity.getId() == null || activity.getId().trim().length() == 0) activity.setId(createUniqueAcitivyID());
                activities.put(activity.getId(), activity);
            }
        }
    }

    protected String createUniqueAcitivyID(){
        return "executableActivity" + ResourceUtils.generateUniqueID();
    }

    public Iterator<Entry<String,String>> getDefinitionIterator(){
        return definitions.entrySet().iterator();
    }

    public String toTypeXML(String location, String pipeID, String workflowID){
        String xml = "";
        if(getType() != null) xml += "<goal id=\"" + getId() + "\" type=\"" + getType().getLocalName() + "\" location=\"" + location + "\" pipeID=\"" + pipeID + "\" workflowID=\"" + workflowID + "\">";
        Enumeration<ExecutableActivity> enumeration = activities.elements();
        while(enumeration.hasMoreElements()){
            ExecutableActivity ea = enumeration.nextElement();
            String res = ea.toTypeXML(location, pipeID, workflowID);
            if(res != null) xml += res + "\n";
        }
        if(getType() != null) xml += "</goal>";
        return xml.length() == 0 ? null : xml;
    }

    public String toXML() {
        String xml = "<goal" + (getId() == null ? "" : " id=\"" + getId() + "\"") + (getType() == null ? "" : " type=\"" + getType().getLocalName() + "\">\n");
        if(incentive != null) xml += incentive.toXML() + "\n";
        Enumeration<ExecutableActivity> enumeration = activities.elements();
        while(enumeration.hasMoreElements()){
            xml += enumeration.nextElement().toXML() + "\n";
        }
        xml += "</goal>";
        return xml;
    }

    public boolean getResult(){
        return result;
    }

    public boolean execute(){
        getLog().logMessage(this, getLog().LEVEL_INFORMATIVE, "Executing.");
        incentiveRunner = new IncentiveRunner();
        result = true;
        new HaltedRunner();
        try{
            incentiveRunner.join();
        } catch(InterruptedException ex){
            ex.printStackTrace();
            return result = false;
        }
        return !isHalted();
    }

    private class IncentiveRunner extends Thread{
        private boolean run;
        private boolean hasIncentive;
        private ExecutableActivityRunner ifTrue;
        private ExecutableActivityRunner ifFalse;

        public IncentiveRunner(){
            run = true;
            hasIncentive = false;
            start();
        }

        public void run(){
            if(incentive == null){
                result = false;
                return;
            }
            while(run){
                if(isHalted()){
                    result = false;
                    return;
                }
                try{
                    if(!incentive.execute()){
                        getLog().logMessage(Goal.this, getLog().LEVEL_ERROR, "Unable to execute incentive.");
                        result = false;
                        return;
                    }
                    getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "Incentive executed.");
                    if(incentive.getResult() && !hasIncentive){
                        getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "Incentives satisfied. Starting activities...");
                        hasIncentive = true;
                        // Run iftrue
                        ifTrue = new ExecutableActivityRunner(this, getActivity(incentive.getExecuteIfTrue()), true);

                    }else if(!incentive.getResult() && hasIncentive){
                        getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "Incentives not satisfied. Stopping activities...");
                        hasIncentive = false;
                        // Stop iftrue
                        ifTrue.activity.halt();
                        try{
                            ifTrue.join();
                        } catch(Exception ex){}
                        // Run iffalse
                        ifFalse = new ExecutableActivityRunner(this, getActivity(incentive.getExecuteIfFalse()), false);
                        try{
                            ifFalse.join();
                        } catch(Exception ex){}
                        //result = ifFalse.execution;
                        //return;
                        ifTrue.activity.clearHalt();
                    }

                } catch(Exception ex){
                    ex.printStackTrace();
                    return;
                }
                try{
                    Thread.sleep(incentive.getInterval());
                } catch(Exception ex){
                    ex.printStackTrace();
                }
            }
        }

        private void runnerCompleted(){
            if(hasIncentive){
                try{
                    run = false;
                    result = ifTrue.execution;
                    getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "Activities stopped.");
                    this.interrupt();
                } catch(Exception ex){}
            }
        }

    }

    private class HaltedRunner extends Thread{
        private boolean run;

        public HaltedRunner(){
            run = true;
            start();
        }

        public void run(){
            getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "HaltedRunner started.");
            while(run){
                try{
                    Thread.sleep(5000);
                } catch(Exception ex){
                    ex.printStackTrace();
                }
                if(incentiveRunner == null || !incentiveRunner.isAlive()){
                    getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "HaltedRunner stopped.");
                    return;
                }
                if(isHalted()) incentiveRunner.interrupt();
            }
            getLog().logMessage(Goal.this, getLog().LEVEL_INFORMATIVE, "HaltedRunner stopped.");
        }

    }

    private class ExecutableActivityRunner extends Thread{
        private boolean execution;
        private ExecutableActivity activity;
        private IncentiveRunner incentiveRunner;
        private boolean callback;

        public ExecutableActivityRunner(IncentiveRunner incentiveRunner, ExecutableActivity activity, boolean callback){
            this.incentiveRunner = incentiveRunner;
            this.activity = activity;
            this.callback = callback;
            start();
        }

        public void run(){
            if(activity != null) execution = activity.execute();
            if(callback) incentiveRunner.runnerCompleted();
        }

    }

}
