package edu.ntnu.cikmr.workflow;

import java.util.*;

import org.w3c.dom.*;

/**
 *
 * <p>Title: Incentive statment</p>
 *
 * <p>Description: Checks if the conditions to start the workflows are fulfilled.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class Incentive extends Condition{
    private int interval;
    private Goal parent;
    private String ifTrue;
    private String ifFalse;

    /**
     * Returns the id to execute if true
     *
     * @return id
     */
    public String getExecuteIfTrue(){
        return ifTrue;
    }

    /**
     * Returns the id to execute if false
     *
     * @return id
     */
    public String getExecuteIfFalse(){
        return ifFalse;
    }

    /**
     * Sets the id to execute if true
     *
     * @param s id
     */
    public void setExecuteIfTrue(String s){
        ifTrue = s;
    }

    /**
     * Sets the id to execute if false.
     *
     * @param s id
     */
    public void setExecuteIfFalse(String s){
        ifFalse = s;
    }

    /**
     * Returns the incentive execution interval.
     *
     * @return interval
     */
    public int getInterval(){
        return interval;
    }

    /**
     * Returns the id to execute if false
     *
     * @return id
     */
    public String getIfFalse() {
        return ifFalse;
    }

    /**
     * Returns the id to execute if true
     *
     * @return id
     */
    public String getIfTrue() {
        return ifTrue;
    }

    /**
     * Returns the execution interval of the incentive.
     *
     * @param interval
     */
    public void setInterval(int interval){
        this.interval = interval;
    }

    public void fromXML(Node n) {
        NamedNodeMap nodeMap = n.getAttributes();
        Node interval = nodeMap.getNamedItem("interval");
        try{
            this.interval = interval == null ? 10000 : Integer.parseInt(interval.getNodeValue(), 10);
        } catch(Exception ex){
            ex.printStackTrace();
            this.interval = 10000;
        }
        super.fromXML(n);
        setExecuteIfFalse(getIffalse());
        setExecuteIfTrue(getIftrue());
        setIftrue(null);
        setIffalse(null);
    }

    public String toXML() {
        String xml = "<incentive" + (ifTrue == null ? "" : " iftrue=\"" + ifTrue + "\"") + (ifFalse == null ? "" : " iffalse=\"" + ifFalse + "\"") + " binding=\"" + (getAndBinding() ? "and" : "or") + "\"";
        if(super.getStatementCount() > 0){
            xml += ">\n";
            Iterator<Statement> iterator = super.getStatementIterator();
            while(iterator.hasNext()){
                xml += iterator.next().toXML() + "\n";
            }
            xml += "</incentive>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public void setParent(Goal goal){
        Activity incentive = new Activity();
        incentive.setParent(goal);
        this.setParent(incentive);
    }

    /**
     * Sets the id to execute if false
     *
     * @param s id
     */
    public void setIfFalse(String ifFalse) {
        this.ifFalse = ifFalse;
    }

    /**
     * Sets the id to execute if true
     *
     * @param s id
     */
    public void setIfTrue(String ifTrue) {
        this.ifTrue = ifTrue;
    }

}
