package edu.ntnu.cikmr.workflow;

import java.util.ArrayList;
import java.util.Iterator;

import org.w3c.dom.*;
import edu.ntnu.cikmr.services.resources.Resource;
import edu.ntnu.cikmr.services.resources.ResourceMethod;
import edu.ntnu.cikmr.services.resources.*;

/**
 *
 * <p>Title: Invoke statement</p>
 *
 * <p>Description: Invokes a method on a resource.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Ruds
 * @version 1.0
 */
public class Invoke extends Statement{
    private String method;
    private String name;
    private boolean result;
    private ArrayList<String> parameters;
    private UseResource useResource;

    public Invoke(){
        parameters = new ArrayList<String>();
    }

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node method = nodeMap.getNamedItem("method");
        this.method = method == null ? null : method.getNodeValue();
        Node name = nodeMap.getNamedItem("name");
        this.name = name == null ? null : name.getNodeValue();

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            if(childNode.getNodeName().equalsIgnoreCase("parameter")) getXMLParameter(childNode);
        }
    }

    private void getXMLParameter(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node value = nodeMap.getNamedItem("value");
        if(value != null) parameters.add(value.getNodeValue());
    }

    public String toXML(){
        String xml = "<invoke method=\"" + method + "\"";
        xml += name == null ? "" : " name=\"" + name + "\"";
        if(parameters.size() > 0){
            xml += ">\n";
            Iterator<String> iterator = parameters.iterator();
            while(iterator.hasNext()){
                String value = iterator.next();
                xml += "<parameter value=\"" + value + "\"/>\n";
            }
            xml += "</invoke>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public boolean execute(){
        result = false;
        Resource resource = useResource.getResource();
        if(resource == null){
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "No resource.");
            return result;
        }
        try{
            ResourceMethod resourceMethod = resource.getResourceDescription().getMethod(resolve(method));
            if(resourceMethod == null){
                getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "No method.");
                return result;
            }
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing: " + resourceMethod.getMethodName());
            String[] params = getParameters();
            InvocationResult invocationResult = resourceMethod.invokeMethod(params);
            if(name != null) getParent().getParent().setDefinition(resolve(name), invocationResult.getReturnValue().toString());
            result = true;
            if(!invocationResult.getDataType().equalsIgnoreCase("void")) getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + invocationResult.getReturnValue());
        } catch(Exception ex){
            ex.printStackTrace();
            result = false;
        }
        return result;
    }

    private String[] getParameters(){
        String[] preparedParameters = new String[parameters.size()];
        for(int i = 0; i < preparedParameters.length; i++){
            preparedParameters[i] = resolve(parameters.get(i));
        }
        return preparedParameters;
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Returns the name of the method to invoke.
     *
     * @return method name
     */
    public String getMethod(){
        return method;
    }

    /**
     * Returns the name of the variable to store the result in.
     *
     * @return variable name
     */
    public String getName(){
        return name;
    }

    public UseResource getUseResource(){
        return useResource;
    }

    /**
     * Sets the name of the method to invoke.
     *
     * @param method method name
     */
    public void setMethod(String method){
        this.method = method;
    }

    /**
     * Sets the name of the variable to store the result in.
     *
     * @param name variable name
     */
    public void setName(String name){
        this.name = name;
    }

    public void setUseResource(UseResource useResource){
        this.useResource = useResource;
    }

    public void clearParameters(){
        parameters.clear();
    }

    public void addParameter(String value, int index){
        parameters.add(index, value);
    }

    public void addParameter(String value){
        parameters.add(value);
    }

    public void removeParameter(int index){
        parameters.remove(index);
    }

}
