package edu.ntnu.cikmr.workflow;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

/**
 *
 * <p>Title: IsDefined statement</p>
 *
 * <p>Description: Checks if a variable is defined.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class IsDefined extends Statement{
    private String name;
    private boolean result;

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node name = nodeMap.getNamedItem("name");
        this.name = name == null ? null : name.getNodeValue();
    }

    public String toXML(){
        return "<isDefined name=\"" + name + "\"/>";
    }

    public boolean execute(){
        result = false;
        String vName = resolve(name);
        boolean isDefined = false;
        isDefined = getParent().getParent().hasDefinition(vName);
        if(isDefined){
            String value = getParent().getParent().getDefinition(vName);
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + vName + "=" + isDefined + " (" + value + ")");
            return result = isDefined;
        }
        isDefined = getParent().getParent().hasResource(resolve(name));
        result = isDefined;
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + vName + "=" + result);
        return true;
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Returns the name of the variable to check.
     *
     * @return variable name
     */
    public String getName(){
        return name;
    }

    /**
     * Sets the name of the variable to check.
     *
     * @param name variable name
     */
    public void setName(String name){
        this.name = name;
    }

}
