package edu.ntnu.cikmr.workflow;

import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

/**
 *
 * <p>Title: SetDefined</p>
 *
 * <p>Description: Sets the value or creates a variable.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class SetDefined extends Statement{
    private String name;
    private String value;
    private boolean result;

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node name = nodeMap.getNamedItem("name");
        this.name = name == null ? null : name.getNodeValue();
        Node value = nodeMap.getNamedItem("value");
        this.value = value == null ? null : value.getNodeValue();
    }

    public String toXML(){
        return "<setDefined name=\"" + name + "\" value=\"" + value + "\"/>";
    }

    public boolean execute(){
        try{
            String vName = resolve(name);
            String vValue = resolve(value);
            getParent().getParent().setDefinition(vName, vValue);
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Result: " + vName + "=" + vValue);
            result = true;
        } catch(Exception ex){
            ex.printStackTrace();
            getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Execution failed.");
            result = false;
        }
        return result;
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Returns the value to set.
     *
     * @return value
     */
    public String getValue(){
        return value;
    }

    /**
     * Returns the name of the variable.
     *
     * @return variable name.
     */
    public String getName(){
        return name;
    }

    /**
     * Sets the variable value.
     *
     * @param value value
     */
    public void setValue(String value){
        this.value = value;
    }

    /**
     * Sets the name of the variable to set.
     *
     * @param name variable name
     */
    public void setName(String name){
        this.name = name;
    }

}
