package edu.ntnu.cikmr.workflow;

import java.util.*;

import edu.ntnu.cikmr.services.resources.*;
import org.w3c.dom.*;

/**
 *
 * <p>Title: UseResource statement</p>
 *
 * <p>Description: Uses a resource stored in a variable.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class UseResource extends Statement{
    private String name;
    private List<Invoke> invokes;
    private boolean result;

    public UseResource(){
        invokes = Collections.synchronizedList(new ArrayList<Invoke>());
    }

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node name = nodeMap.getNamedItem("name");
        this.name = name == null ? null : name.getNodeValue();

        NodeList nodeList = n.getChildNodes();
        for(int i = 0; i < nodeList.getLength(); i++){
            Node childNode = nodeList.item(i);
            if(childNode.getNodeName().equalsIgnoreCase("invoke")) fromXMLInvoke(childNode);
        }
    }

    private void fromXMLInvoke(Node n){
        Invoke i = new Invoke();
        i.setUseResource(this);
        i.setParent(getParent());
        i.fromXML(n);
        invokes.add(i);
    }

    public String toXML(){
        String xml = "<useResource name=\"" + name + "\"";
        if(invokes.size() > 0){
            xml += ">\n";
            Iterator<Invoke> iterator = invokes.iterator();
            while(iterator.hasNext()){
                xml += iterator.next().toXML() + "\n";
            }
            xml += "</useResource>";
        }else{
            xml += "/>";
        }
        return xml;
    }

    public boolean execute(){
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing: " + invokes.size() + " methods.");
        result = false;
        Iterator<Invoke> iterator = invokes.iterator();
        while(iterator.hasNext()){
            Invoke invoke = iterator.next();
            if(!invoke.execute()){
                return result;
            }
        }
        return result = true;
    }

    public boolean getResult(){
        return result;
    }

    /**
     * Returns the name of the variable containing the resource to look up.
     *
     * @return resource variable name
     */
    public String getName(){
        return name;
    }

    /**
     * Returns the list of Invokes.
     *
     * @return invokes
     */
    public List getInvokes() {
        return invokes;
    }

    /**
     * Sets the name of the resource variable to look up.
     *
     * @param name resource variable name
     */
    public void setName(String name){
        this.name = name;
    }

    /**
     * Sets the list of Invokes.
     *
     * @param invokes list of Invokes.
     */
    public void setInvokes(List invokes) {
        this.invokes = invokes;
    }

    public Resource getResource(){
        return getParent().getParent().getResource(resolve(name));
    }

}
