package edu.ntnu.cikmr.workflow;

import org.w3c.dom.*;

/**
 *
 * <p>Title: Wait statement</p>
 *
 * <p>Description: Wait a configurable amount of milliseconds.</p>
 *
 * <p>Copyright: Copyright (c) 2007</p>
 *
 * @author Christian Indahl and Kjell M. Rud
 * @version 1.0
 */
public class Wait extends Statement{
    private int milliseconds;
    private boolean result;

    public void fromXML(Node n){
        NamedNodeMap nodeMap = n.getAttributes();
        Node milliseconds = nodeMap.getNamedItem("milliseconds");
        try {
            this.milliseconds = Integer.parseInt(milliseconds.getNodeValue(), 10);
        } catch (Exception ex) {
            ex.printStackTrace();
            this.milliseconds = 10000;
        }
    }

    public boolean execute() {
        getParent().getLog().logMessage(this, getParent().getLog().LEVEL_INFORMATIVE, "Executing: " + milliseconds);

        int iterations = milliseconds / 1000;
        int rest = milliseconds % 1000;

        result = false;
        try {
            for (int i = 0; i < iterations; i++) {
                Thread.sleep(1000);
                if(getParent().isHalted()) return result = true;
            }
            Thread.sleep(rest);
            result = true;
        } catch (Exception ex) {
            ex.printStackTrace();
            result = false;
        }
        return result;
    }

    public boolean getResult() {
        return result;
    }

    /**
     * Returns the amount of milliseconds to wait.
     *
     * @return milliseconds
     */
    public int getMilliseconds() {
        return milliseconds;
    }

    public String toXML() {
        return "<wait milliseconds=\"" + milliseconds + "\">";
    }

    /**
     * Sets the amount of milliseconds to wait.
     *
     * @param milliseconds the amount of milliseconds to wait.
     */
    public void setMilliseconds(int milliseconds) {
        this.milliseconds = milliseconds;
    }

}
