package edu.ntnu.cikmr.workflowenactment;

import java.util.*;

import javax.swing.event.EventListenerList;

import edu.ntnu.cikmr.log.Log;

public abstract class WorkflowLog extends Log{
    private List<WorkflowLogEntry> entries;
    private EventListenerList listenerList;
    private int threshold;

    public WorkflowLog(){
        entries = Collections.synchronizedList(new ArrayList<WorkflowLogEntry>());
        listenerList = new EventListenerList();
        threshold = 100;
    }

    public void setThreshold(int size){
        threshold = size;
    }

    public int getThreshold(){
        return threshold;
    }

    protected void addEntry(WorkflowLogEntry entry){
        entries.add(0, entry);
        trim();
        fireLogUpdated();
    }

    protected void addEntries(WorkflowLogEntry[] entries){
        for(int i = 0; i < entries.length; i++) this.entries.add(0, entries[i]);
        trim();
        fireLogUpdated();
    }

    protected void trim(){
        while(entries.size() > threshold) entries.remove(entries.size() - 1);
    }

    protected WorkflowLogEntry[] getEntries(){
        return entries.toArray(new WorkflowLogEntry[entries.size()]);
    }

    public WorkflowLogEntry getEntry(int index){
        return entries.get(index);
    }

    public int getEntriesCount(){
        return entries.size();
    }

    public void addWorkflowLogListener(WorkflowLogListener l){
        listenerList.add(WorkflowLogListener.class, l);
    }

    public void removeWorkflowLogListener(WorkflowLogListener l){
        listenerList.remove(WorkflowLogListener.class, l);
    }

    protected void fireLogUpdated(){
        EventObject event = new EventObject(this);
        WorkflowLogListener[] listeners = listenerList.getListeners(WorkflowLogListener.class);
        for(int i = listeners.length - 1; i >= 0; i--) listeners[i].logUpdated(event);
    }

}
